
//  Created by VaxSoft
//  Copyright © 2025 VaxSoft. All rights reserved.

#import "cAddressBook.h"
#import <AddressBook/AddressBook.h>
#import <AddressBookUI/AddressBookUI.h>
#import <Contacts/Contacts.h>

@implementation cAddressBook

- (id) init;
{
    self = [super init];
    return self;
}

- (void) dealloc;
{

}

////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////

+ (void) AskAcessToContacts;
{
    CNContactStore* pContactStore = [[CNContactStore alloc] init];
    [pContactStore requestAccessForEntityType:CNEntityTypeContacts completionHandler:^(BOOL granted, NSError * _Nullable error)
     {
         if (granted == YES)
         {
             
         }
     }];
}

- (Boolean) GetAccess;
{
    CNEntityType entityType = CNEntityTypeContacts;
    
    if([CNContactStore authorizationStatusForEntityType:entityType] == CNAuthorizationStatusNotDetermined)
    {
        [cAddressBook AskAcessToContacts];
        return false;
    }
    else if( [CNContactStore authorizationStatusForEntityType:entityType]== CNAuthorizationStatusAuthorized)
    {
        return true;
    }
    else
    {
        return false;
    }
    
    return false;
}

- (NSString*) ReadContactName :(NSString*) sContactId;
{
    if(![self GetAccess])
        return @"";
    
    CNContactStore* pContactStore = [[CNContactStore alloc] init];
    CNContactFetchRequest* request = [[CNContactFetchRequest alloc] initWithKeysToFetch:@[CNContactIdentifierKey, [CNContactFormatter descriptorForRequiredKeysForStyle:CNContactFormatterStyleFullName]]];
    
    NSError* Error;
    NSMutableArray* aContacts = [NSMutableArray new];
    
    BOOL bSuccess = [pContactStore enumerateContactsWithFetchRequest:request error:&Error usingBlock:^(CNContact *contact, BOOL *stop)
    {
        [aContacts addObject :contact];
    }];
    
    if (!bSuccess)
    {
        return @"";
    }

    if(aContacts.count <= 0) return @"";
    
    for (CNContact* pContact in aContacts)
    {
        if(![sContactId isEqualToString :pContact.identifier])
            continue;
        
        CNContactFormatter* formatter = [[CNContactFormatter alloc] init];
        NSString* sContactName = [formatter stringFromContact :pContact];
        
        return sContactName;
    }
    
    return @"";
}

- (Boolean) IsContactExist :(NSString*) sContactId;
{
    NSString* sContactName = [self ReadContactName :sContactId];
    if(sContactName.length == 0) return false;
    
    return true;
}

- (NSString*) FindContactNameWithPhoneNo :(NSString*) sPhoneNo;
{
    NSMutableString* sContactName = [NSMutableString new];
    NSMutableString* sContactId = [NSMutableString new];
    
    if(![self FindContactWithPhoneNo :sPhoneNo :&sContactName :&sContactId])
        return @"";
    
    return [sContactName copy];
}

+ (NSString*) FilterPhoneNo :(NSString*) sPhoneNo;
{
    NSMutableCharacterSet* PhoneNoCharacterSet = [NSMutableCharacterSet characterSetWithCharactersInString:@"+"];
    [PhoneNoCharacterSet formUnionWithCharacterSet:[NSCharacterSet decimalDigitCharacterSet]];
    
    sPhoneNo = [[sPhoneNo componentsSeparatedByCharactersInSet:[PhoneNoCharacterSet invertedSet]] componentsJoinedByString:@""];
    
    return [sPhoneNo copy];
}

- (Boolean) FindContactWithPhoneNo :(NSString*) sPhoneNo :(NSMutableString**) sResultName :(NSMutableString**) sResultContactId;
{
    if(![self GetAccess])
        return false;
    
    CNContactStore* pContactStore = [[CNContactStore alloc] init];
    CNContactFetchRequest* request = [[CNContactFetchRequest alloc] initWithKeysToFetch:@[CNContactPhoneNumbersKey, [CNContactFormatter descriptorForRequiredKeysForStyle:CNContactFormatterStyleFullName]]];
    
    NSError* Error;
    NSMutableArray* aContacts = [NSMutableArray new];
    
    BOOL bSuccess = [pContactStore enumerateContactsWithFetchRequest:request error:&Error usingBlock:^(CNContact *contact, BOOL *stop)
    {
        [aContacts addObject :contact];
    }];
    
    if (!bSuccess)
    {
        return false;
    }

    for (CNContact* pContact in aContacts)
    {
        for (CNLabeledValue *label in pContact.phoneNumbers)
        {
            NSString* sContactPhone = [label.value stringValue];
            if ([sContactPhone length] <= 0) continue;
            
            sContactPhone = [cAddressBook FilterPhoneNo :sContactPhone];
            sPhoneNo = [cAddressBook FilterPhoneNo :sPhoneNo];
                
            if(![sContactPhone isEqualToString: sPhoneNo])
                continue;
                
            NSString* sContactId = pContact.identifier;
            
            CNContactFormatter* formatter = [[CNContactFormatter alloc] init];
            NSString* sContactName = [formatter stringFromContact :pContact];
                
            [*sResultName setString :sContactName];
            [*sResultContactId setString :sContactId];
                
            return true;
        }
    }
    
    return false;
}


@end




