
//  Created by VaxSoft
//  Copyright © 2025 VaxSoft. All rights reserved.

#import "cAddressBookView.h"
#import "cAddressBook.h"

@implementation cAddressBookView
@synthesize mAddressBookDelegate;

- (id)initWith:(id) iDelegate;
{
    self = [super init];
    
    mAddressBookDelegate = iDelegate;
    
    mAddressBook = nil;
    m_bAccessOnlyContacts = false;
    
    return self;
}

- (void)dealloc
{
    [self HideAddressBookUI];
}

- (void)didReceiveMemoryWarning
{
}

- (void) contactPicker:(CNContactPickerViewController*)picker didSelectContact:(CNContact *)contact
{
    [self OnSelectedContact :contact.identifier];
}

- (void) contactPicker:(CNContactPickerViewController *)picker didSelectContactProperty:(CNContactProperty *)contactProperty;
{
    //CNContact* contact = contactProperty.contact;
    
    CNPhoneNumber* contactPhone = contactProperty.value;
    NSString* phoneNumber = contactPhone.stringValue;
    
    [self OnSelectedPhoneNo :phoneNumber];
    
}

- (void)contactPickerDidCancel:(CNContactPickerViewController *)picker
{

}

//////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////

- (void) ChangeNavigationBar;
{
    [[UINavigationBar appearance] setTintColor:[UIColor darkGrayColor]];
}

- (void) ShowContacts :(UIViewController*) pParent :(bool) bAccessOnlyContacts;
{
    [self ChangeNavigationBar];
    [self ShowAddressBookUI :pParent :bAccessOnlyContacts];
}

- (void) OnAddressBookAccess:(UIViewController*) pParent;
{
    if(!m_bGranted)
    {
        [self CloseAddressBook];
        return;
    }
    
    [pParent presentViewController:mAddressBook animated:YES completion:nil];
}

- (Boolean) ShowAddressBookUI :(UIViewController*) pParent :(bool) bAccessOnlyContacts;
{
    mAddressBook = [[CNContactPickerViewController alloc] init];
    mAddressBook.delegate = self;
    
    mAddressBook.displayedPropertyKeys = @[CNContactPhoneNumbersKey];
    
    m_bAccessOnlyContacts = bAccessOnlyContacts;
    
    [pParent presentViewController :mAddressBook animated:YES completion:nil];
    
        if(bAccessOnlyContacts == false)
           mAddressBook.predicateForSelectionOfContact = [NSPredicate predicateWithValue:false];
    
    return true;
}

- (void) HideAddressBookUI;
{
    mAddressBook = NULL;
}

- (void) OnDimissCompleted;
{
    [self HideAddressBookUI];
}

- (void) SetDefaultNavigationBar;
{
    /*
    UIImage *navbar = [UIImage imageNamed :@"navbar.png"];
    
    [[UINavigationBar appearance] setBackgroundImage:navbar forBarMetrics:UIBarMetricsDefault];
    [[UINavigationBar appearance] setTintColor:[UIColor whiteColor]];
    
    NSDictionary *textTitleOptions = [NSDictionary dictionaryWithObjectsAndKeys:[UIColor whiteColor], NSForegroundColorAttributeName, [UIColor whiteColor], NSForegroundColorAttributeName, nil];
    
    [[UINavigationBar appearance] setTitleTextAttributes:textTitleOptions];
     */
}

- (void) CloseAddressBook;
{
    [self SetDefaultNavigationBar];
    
    [mAddressBook dismissViewControllerAnimated:YES completion:^
     {
         [self performSelectorOnMainThread:@selector(OnDimissCompleted) withObject:nil waitUntilDone:false];
     }];
}

////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////

- (void) OnSelectedPhoneNo :(NSString*) sPhoneNo;
{
    if(mAddressBookDelegate == NULL)
        return;
    
    if (![mAddressBookDelegate respondsToSelector:@selector(OnAddressBookSelectedPhoneNo:)])
        return;
    
    [mAddressBookDelegate OnAddressBookSelectedPhoneNo :sPhoneNo];
    [self CloseAddressBook];
}

- (void) OnSelectedContact :(NSString*) sContactId;
{
    if(mAddressBookDelegate == NULL)
        return;
    
    if (![mAddressBookDelegate respondsToSelector:@selector(OnAddressBookSelectedContact::)])
        return;
    
    cAddressBook* pBook = [cAddressBook new];
    NSString* sTitle = [pBook ReadContactName :sContactId];
    
    [mAddressBookDelegate OnAddressBookSelectedContact :sContactId :sTitle];
    [self CloseAddressBook];
}

////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////

@end

