
//  Created by VaxSoft
//  Copyright © 2025 VaxSoft. All rights reserved.

#import "AppDelegate.h"
#import "cDataSQL.h"
#import "cVaxPhoneSIP.h"
#import "cAddressBook.h"
#import "cRecentTableView.h"
#import "cChatContactTableView.h"
#import "cSettingsTableView.h"
#import "cSettingMicSpkTableView.h"
#import "cSettingNetworkTableView.h"
#import "cAccountInfoTableView.h"

#import "cSettingVoiceChangerTableView.h"
#import "cSettingVoiceCodecTableView.h"
#import "cSettingRingToneTableView.h"
#import "cSettingVideoCodecTableView.h"
#import "cSettingDigitDTMFTableView.h"
#import "cVideoView.h"

#import "cSettingVideoQualityTableView.h"
#import "cSettingVideoBitrateTableView.h"

#import "cStoreRecents.h"
#import "cStoreChatContacts.h"
#import "cStoreChatMsgs.h"

#import <UserNotifications/UserNotifications.h>

@interface AppDelegate ()
@end

static int m_nMainAppMissedCount = 0;
static Boolean m_bMainAppBackground = true;

@implementation AppDelegate

+ (void) LoadDataAll;
{
    [cVideoView OnStartedAPP];
    [cChatContactTableView OnStartedAPP];
}

+ (void) ResetSettings;
{
    [cVideoView ResetData];
    
    [cSettingMicSpkTableView ResetData];
    [cSettingNetworkTableView ResetData];
      
    [cSettingVoiceChangerTableView ResetData];
    [cSettingVoiceCodecTableView ResetData];
    
    [cSettingRingToneTableView ResetData];
    [cSettingVideoCodecTableView ResetData];
    [cSettingDigitDTMFTableView ResetData];
    
    [cSettingVideoQualityTableView ResetData];
    [cSettingVideoBitrateTableView ResetData];
}

////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////

+ (void) PostChatMsgRecvText;
{
    if(!m_bMainAppBackground)
        return;
    
    [AppDelegate SetAppMissedCount :-1];
}

+ (void) PostInboundCallHistory;
{
    if(!m_bMainAppBackground)
        return;
    
    [AppDelegate SetAppMissedCount :-1];
}

+ (void) SetAppMissedCount :(int) nValue;
{
    if(nValue == 0)
    {
        m_nMainAppMissedCount = 0;
        
    }
    else
    {
        m_nMainAppMissedCount++;
    }
    
    [[UIApplication sharedApplication] setApplicationIconBadgeNumber :m_nMainAppMissedCount];
}

////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////

- (BOOL)application:(UIApplication *)application didFinishLaunchingWithOptions:(NSDictionary *)launchOptions
{
    //[[UIApplication sharedApplication] setStatusBarHidden :NO];
    
    ///////////////////////////////////////////////
    
    [cDataSQL CreateFileDB];
    
    cStoreRecents* pRecentDB = [cStoreRecents new];
    [pRecentDB CreateTable];
    
    cStoreChatContacts* pChatContactDB = [cStoreChatContacts new];
    [pChatContactDB CreateTable];
    
    cStoreChatMsgs* pChatMsgDB = [cStoreChatMsgs new];
    [pChatMsgDB CreateTable];
    
    ///////////////////////////////////////////////
    
    mVaxVoip = [cVaxPhoneSIP new];
    
    [cAddressBook AskAcessToContacts];
    [AppDelegate LoadDataAll];
    
    [self AskPermissions];
    
    [AppDelegate SetAppMissedCount :0];
    m_bMainAppBackground = false;
    
    return YES;
}

- (void) AskPermissions;
{
    // iOS 10+
    
    UNUserNotificationCenter* center = [UNUserNotificationCenter currentNotificationCenter];
    
    [center getNotificationSettingsWithCompletionHandler:^(UNNotificationSettings * _Nonnull settings)
     {
        switch (settings.authorizationStatus)
        {
            case UNAuthorizationStatusNotDetermined:
            {
                //center.delegate = self;

                UNAuthorizationOptions options = UNAuthorizationOptionSound | UNAuthorizationOptionAlert | UNAuthorizationOptionBadge;
                
                [center requestAuthorizationWithOptions :options completionHandler:^(BOOL granted, NSError * _Nullable error)
                 {
                     if(granted)
                     {
                        dispatch_async(dispatch_get_main_queue(), ^
                        {
                             [[UIApplication sharedApplication] registerForRemoteNotifications];
                         });
                     }
                     else
                     {
                         // Notify user to enable push notification permission in settings
                     }
                 }];
                break;
            }
            
            case UNAuthorizationStatusDenied:
            {
                // Notify user to enable push notification permission in settings
                break;
            }
           
            case UNAuthorizationStatusAuthorized:
            {
                dispatch_async(dispatch_get_main_queue(), ^
                {
                     [[UIApplication sharedApplication] registerForRemoteNotifications];
                });
                
                break;
            }
            default:
                break;
        }
    }];
}


- (void)applicationWillResignActive:(UIApplication *)application
{
    // Sent when the application is about to move from active to inactive state. This can occur for certain types of temporary interruptions (such as an incoming phone call or SMS message) or when the user quits the application and it begins the transition to the background state.
    // Use this method to pause ongoing tasks, disable timers, and throttle down OpenGL ES frame rates. Games should use this method to pause the game.
    
    
}

- (void)applicationDidEnterBackground:(UIApplication *)application
{
    m_bMainAppBackground = true;
    
    // Use this method to release shared resources, save user data, invalidate timers, and store enough application state information to restore your application to its current state in case it is terminated later.
    // If your application supports background execution, this method is called instead of applicationWillTerminate: when the user quits.
}

- (void)applicationWillEnterForeground:(UIApplication *)application
{
    // Called as part of the transition from the background to the inactive state; here you can undo many of the changes made on entering the background.
    
    [AppDelegate SetAppMissedCount :0];
    m_bMainAppBackground = false;
}

- (void)applicationDidBecomeActive:(UIApplication *)application
{
    // Restart any tasks that were paused (or not yet started) while the application was inactive. If the application was previously in the background, optionally refresh the user interface.
}

- (void)applicationWillTerminate:(UIApplication *)application
{
    // Called when the application is about to terminate. Save data if appropriate. See also applicationDidEnterBackground:.
}

@end
