
//  Created by VaxSoft
//  Copyright © 2025 VaxSoft. All rights reserved.

#import <AVFoundation/AVFoundation.h>
#import "cVaxCallKit.h"

@implementation cVaxCallKit
@synthesize mCallKitDelegate;

- (id)initWith:(id) iDelegate;
{
    self = [super init];
    
    mCallKitDelegate = iDelegate;
    mbCallKitUI = false;
    
    m_hUUID = [NSUUID UUID];
    
    return self;
}

- (void) dealloc;
{
   
}

////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////

- (void) Destroy;
{
    if(mbCallKitUI == false)
        return;
    
    mbCallKitUI = false;
    
    [mProviderCX reportCallWithUUID :m_hUUID endedAtDate:[NSDate date] reason:CXCallEndedReasonRemoteEnded];
    
    mUpdateCX = nil;
    mCallCtrl = nil;
    mProviderCX = nil;
}

- (Boolean) Create;
{
    [self Destroy];
    
    if (NSFoundationVersionNumber <= 1300.000000)
    {
        NSLog(@"Call-kit: Not supported.");
        return false;
    }
    
    m_hUUID = [NSUUID UUID];
    mCallCtrl = [[CXCallController alloc] init];
    
    NSLog(@"Call-kit supported platform.");
        
    mConfigCX = [[CXProviderConfiguration alloc] initWithLocalizedName:@"vax"];
    mConfigCX.maximumCallGroups = 1;
    mConfigCX.maximumCallsPerCallGroup = 1;
    mConfigCX.supportsVideo = false;
        
    mProviderCX = [[CXProvider alloc] initWithConfiguration:mConfigCX];
    [mProviderCX setDelegate:self queue:nil];
    
    mbCallKitUI = true;
    return true;
}

- (Boolean) ShowUI :(NSString*) sCallerName :(NSString*) sCallerId;
{
    if(mbCallKitUI == true)
        return true;
    
    if(![self Create])
        return false;
    
    NSString* sCallerInfo = sCallerId;
    
    if([sCallerName isEqualToString: sCallerId] == false)
    {
        sCallerInfo = [NSString stringWithFormat:@"(%@) %@", sCallerName, sCallerId];
    }
    
    mUpdateCX = [[CXCallUpdate alloc] init];
    mUpdateCX.remoteHandle = [[CXHandle alloc] initWithType:CXHandleTypeGeneric value:@"VaxPhoneSDK"];
    
    mUpdateCX.localizedCallerName = sCallerInfo;
    mUpdateCX.supportsHolding = YES;
    mUpdateCX.supportsGrouping = NO;
    mUpdateCX.supportsUngrouping = NO;
    mUpdateCX.supportsDTMF = YES;
    mUpdateCX.hasVideo = NO;
        
    [mProviderCX setConfiguration:mConfigCX];
    
    [mProviderCX reportNewIncomingCallWithUUID:m_hUUID update:mUpdateCX completion:^(NSError * _Nullable error)
    {
        if (error)
        {
            NSLog(@"Call-Kit Failure");
            //[self Destroy];
        }
    }];
    
    return true;
}

- (void) HideUI;
{
    [self Destroy];
}

#ifdef __IPHONE_10_0
- (void)provider:(CXProvider *)provider performStartCallAction:(CXStartCallAction *)action;
{
}

//answer
- (void)provider:(CXProvider *)provider performAnswerCallAction:(CXAnswerCallAction *)action;
{
    NSLog(@"performAnswerCallAction_%@", action.callUUID);
    
    AVAudioSession* vSession = [AVAudioSession sharedInstance];
    [vSession setCategory:AVAudioSessionCategoryPlayAndRecord withOptions:AVAudioSessionCategoryOptionAllowBluetooth error: nil];
    
    //[vSession setCategory:AVAudioSessionCategoryPlayAndRecord withOptions:AVAudioSessionCategoryOptionAllowBluetooth error: nil];
       
    [action fulfill];
    
    //[action fail];
}

////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////

- (void)provider:(CXProvider *)provider didActivateAudioSession:(AVAudioSession *)audioSession;
{
    if (![mCallKitDelegate respondsToSelector:@selector(OnVaxAnswerCallKit)])
        return;
    
    [mCallKitDelegate OnVaxAnswerCallKit];
}

- (void)provider:(CXProvider *)provider performEndCallAction:(CXEndCallAction *)action;
{
    if (![mCallKitDelegate respondsToSelector:@selector(OnVaxEndCallKit)])
        return;
    
    [mCallKitDelegate OnVaxEndCallKit];
    [action fulfill];
}

- (void)provider:(CXProvider *)provider didDeactivateAudioSession:(AVAudioSession *)audioSession;
{
    NSLog(@"didDeactivateAudioSession");
}

////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////

- (void)provider:(CXProvider *)provider performSetHeldCallAction:(CXSetHeldCallAction *)action;
{
    if (![mCallKitDelegate respondsToSelector:@selector(OnVaxHoldCallKit)])
        return;
    
    [mCallKitDelegate OnVaxHoldCallKit];
}

- (void)provider:(CXProvider *)provider performPlayDTMFCallAction:(CXPlayDTMFCallAction *)action;
{
    if (![mCallKitDelegate respondsToSelector:@selector(OnVaxDigitCallKit)])
        return;
    
    [mCallKitDelegate OnVaxDigitCallKit];
}

////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////

- (void)providerDidReset:(nonnull CXProvider *)provider
{
    
}

#endif


@end
