
//  Created by VaxSoft
//  Copyright © 2025 VaxSoft. All rights reserved.

#import <UIKit/UIAlertView.h>
#import "cVaxPhoneRing.h"

#import "cAVAudioPlayer.h"
#import "cSettingRingToneTableView.h"

static cVaxPhoneRing* mSelf = NULL;

@implementation cVaxPhoneRing

- (id) init;
{
    self = [super init];
    mSelf = self;
    
    mVaxAgentSIP = [[cVaxUserAgentSIP alloc] initWith :self];
    
    m_nRingToneId = VAX_RING_TONE_DIGITAL_RAIN;
    
    mRingToneDATA = NULL;
    mRingTonePLAY = NULL;
    
    return self;
}

- (void) dealloc;
{
    mRingToneDATA = NULL;
    mRingTonePLAY = NULL;
    
    mVaxAgentSIP = NULL;
}

+ (cVaxPhoneRing*) GetSelf;
{
    return mSelf;
}

/////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////

- (Boolean) DialRing :(Boolean) bEnable;
{
    if(bEnable)
    {
        NSString* sFileName = [[NSBundle mainBundle] pathForResource: @"aDialRing" ofType: @"wav"];
        
        return [mVaxAgentSIP DialRingEnable :sFileName];
    }
    
    return [mVaxAgentSIP DialRingDisable];
}

- (Boolean) BusyRing :(Boolean) bEnable;
{
    if(bEnable)
    {
        NSString* sFileName = [[NSBundle mainBundle] pathForResource: @"aBusyRing" ofType: @"wav"];
        
        return [mVaxAgentSIP BusyRingEnable :sFileName];
    }
    
    return [mVaxAgentSIP BusyRingDisable];
}

/////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////

- (void) SetRingTone :(int) nRingToneId;
{
    // For devices, sample code utilizes Call-Kit framework.
    
    #if TARGET_IPHONE_SIMULATOR
        [self LoadRingTone :nRingToneId];
    #endif
}

- (void) MuteRingTone;
{
    // For devices, sample code utilizes Call-Kit framework.
    
    #if TARGET_IPHONE_SIMULATOR
        [self StopRingTone];
    #endif
}

- (void) LoadRingTone :(int) nRingToneId;
{
    NSArray* aFileName = [NSArray arrayWithObjects: @"aRingToneGroovy", @"aRingToneDigitalRain", @"aRingToneMagical", @"aRingToneDejaVu", @"aRingToneOfficePhone", nil];
    
    NSString* sFileName = @"";
    
    if(nRingToneId < 0 || nRingToneId >= [aFileName count])
    {
        m_nRingToneId = VAX_RING_TONE_DIGITAL_RAIN;
        sFileName = @"aRingToneDigitalRain";
    }
    else
    {
        m_nRingToneId = nRingToneId;
        sFileName = [aFileName objectAtIndex:(nRingToneId)];
    }
    
    if(mRingToneDATA)
        mRingToneDATA = NULL;
    
    mRingToneDATA = [[NSMutableData alloc] initWithContentsOfFile:[[NSBundle mainBundle] pathForResource:sFileName ofType:@"mp3"]];
}

/////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////

- (int) GetRingTone;
{
    return m_nRingToneId;
}

/////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////

- (void) StartRingTone;
{
    if(mRingTonePLAY) return;
    
    mRingTonePLAY = [cAVAudioPlayer new];
    [mRingTonePLAY PlayLoadWithData: mRingToneDATA];
    [mRingTonePLAY PlayStart: -1];
}

- (void) StopRingTone;
{
    if(mRingTonePLAY)
    {
        [mRingTonePLAY PlayStop];
        mRingTonePLAY = NULL;
    }
}

/////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////

- (void) OnInitialized;
{
    [cSettingRingToneTableView PostInitialize];
    
    [self BusyRing: true];
    [self DialRing: true];
}

- (void) OnRingToneStarted :(NSString*) sAccountName;
{
    // For devices, sample code utilizes Call-Kit framework.
    
    #if TARGET_IPHONE_SIMULATOR
        [self StartRingTone];
    #endif
}

- (void) OnRingToneEnded :(NSString*) sAccountName;
{
    // For devices, sample code utilizes Call-Kit framework.
    
    #if TARGET_IPHONE_SIMULATOR
        [self StopRingTone];
    #endif
}

@end

