
//  Created by VaxSoft
//  Copyright © 2025 VaxSoft. All rights reserved.

#import <UIKit/UIAlertView.h>
#import "AppDelegate.h"

#import "cVaxPhoneSIP.h"
#import "cVaxPhoneChat.h"

#import "cChatContactTableView.h"

#import "cStoreChatContacts.h"
#import "cStoreChatMsgs.h"

static cVaxPhoneChat* mSelf = NULL;

@implementation cVaxPhoneChat

- (id) init;
{
    self = [super init];
    mSelf = self;
    
    mVaxAgentSIP = [[cVaxUserAgentSIP alloc] initWith :self];
    return self;
}

- (void) dealloc;
{
    mVaxAgentSIP = NULL;
}

+ (cVaxPhoneChat*) GetSelf;
{
    return mSelf;
}

/////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////

- (void) ChatSubscribeContactAll;
{
    [mVaxAgentSIP ChatSubscribeContactAll :ACCOUNT_VAXVOIP];
}

- (Boolean) ChatSetMyStatus :(int) nStatusId;
{
    return [mVaxAgentSIP ChatSetMyStatus :ACCOUNT_VAXVOIP :nStatusId];
}

/////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////

- (void) LoadChatContactAll;
{
    cStoreChatContacts* pStoreDB = [cStoreChatContacts new];
    NSArray* pContactAll = [pStoreDB GetContactAll];
    
    for(long nRecordNo = 0; nRecordNo < [pContactAll count]; nRecordNo++)
    {
        cStoreDataChatContacts* pDataDB = [pContactAll objectAtIndex :nRecordNo];
        
        [mVaxAgentSIP ChatAddContact :ACCOUNT_VAXVOIP :pDataDB->m_sName :pDataDB->m_bPresence];
        [cChatContactTableView PostChatContactAdded :pDataDB->m_sName :pDataDB->m_nMissedCount :pDataDB->m_bPresence];
    }
}

/////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////

- (Boolean) ChatAddContact :(NSString *)sUserName :(Boolean) bPresence;
{
    if(![mVaxAgentSIP ChatAddContact :ACCOUNT_VAXVOIP :sUserName :bPresence])
    {
        return false;
    }
    
    cStoreChatContacts* pStoreDB = [cStoreChatContacts new];
    [pStoreDB AddContact :sUserName :0 :bPresence];

    [cChatContactTableView PostChatContactAdded :sUserName :0 :bPresence];
    return true;
}

- (Boolean) ChatRemoveContact :(NSString*) sUserName;
{
    if(![mVaxAgentSIP ChatRemoveContact :ACCOUNT_VAXVOIP :sUserName])
    {
        return false;
    }
    
    cStoreChatContacts* pChatContactDB = [cStoreChatContacts new];
    [pChatContactDB RemoveContact :sUserName];
    
    cStoreChatMsgs* pChatMsgDB = [cStoreChatMsgs new];
    [pChatMsgDB RemoveMessageAll :sUserName];
    
    return true;
}

/////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////

- (Boolean) ChatSendMessageText :(NSString*) sUserName :(NSString*) sMsgText;
{
    if(![mVaxAgentSIP ChatSendMessageText :ACCOUNT_VAXVOIP :sUserName :sMsgText :false :0])
    {
        return false;
    }
    
    cStoreChatMsgs* pStoreDB = [cStoreChatMsgs new];
    [pStoreDB AddChatMessageText :sUserName :sMsgText :YES];
    
    [cChatContactTableView PostChatMessageText :sUserName :[mVaxAgentSIP ChatFindContact :ACCOUNT_VAXVOIP :sUserName] :sMsgText :YES];
    return true;
}

/////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////

- (void) OnInitialized;
{
    [cChatContactTableView PostInitialize];
}

/////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////

- (void) OnChatContactStatus :(NSString*) sAccountName :(NSString*) sUserName :(int) nStatusId;
{
    [cChatContactTableView PostChatContactStatus :sUserName :nStatusId];
}

- (void) OnChatRecvMsgText :(NSString*) sAccountName :(NSString*) sUserName :(NSString*) sMsgText :(Boolean) bIsChatContact;
{
    if(!bIsChatContact)
    {
        [self ChatAddContact :sUserName :NO];
    }
    
    cStoreChatMsgs* pStoreDB = [cStoreChatMsgs new];
    [pStoreDB AddChatMessageText :sUserName :sMsgText :NO];
    
    [cChatContactTableView PostChatMessageText :sUserName :bIsChatContact :sMsgText :NO];
    [AppDelegate PostChatMsgRecvText];
}

/////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////

@end
