
//  Created by VaxSoft
//  Copyright © 2025 VaxSoft. All rights reserved.

#import <UIKit/UIAlertView.h>
#import "AppDelegate.h"
#import "cVaxPhoneGeneral.h"

#import "cSettingsTableView.h"
#import "cRecentTableView.h"

#import "cVideoView.h"
#import "cStoreRecents.h"
#import "cVaxCallInfo.h"

static cVaxPhoneGeneral* mSelf = NULL;

@implementation cVaxPhoneGeneral

- (id) init;
{
    self = [super init];
    mSelf = self;
    
    mVaxAgentSIP = [[cVaxUserAgentSIP alloc] initWith :self];
    return self;
}

- (void) dealloc;
{
    mVaxAgentSIP = NULL;
}

+ (cVaxPhoneGeneral*) GetSelf;
{
    return mSelf;
}

/////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////

- (Boolean) IsLineBusy;
{
    return [mVaxAgentSIP IsLineBusy :0];
}

- (Boolean) IsLineConnected;
{
    return [mVaxAgentSIP IsLineConnected :0];
}

- (Boolean) IsLineHold;
{
    return [mVaxAgentSIP IsLineHold :0];
}

/////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////

- (Boolean) HoldLine;
{
    return [mVaxAgentSIP HoldLine :0];
}

- (Boolean) UnHoldLine;
{
    return [mVaxAgentSIP UnHoldLine :0];
}

/////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////

- (Boolean) IsSpeakerPhone;
{
    return [mVaxAgentSIP IsSpeakerPhone];
}

- (Boolean) SpeakerPhone :(Boolean) bEnable;
{
    return [mVaxAgentSIP SpeakerPhone :bEnable];
}

/////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////

- (Boolean) IsNetworkAvailable;
{
    return [mVaxAgentSIP IsNetworkAvailable];
}

- (Boolean) NetworkReachability :(Boolean) bEnable;
{
    return [mVaxAgentSIP NetworkReachability :bEnable];
}

/////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////

- (void) OnInitialized;
{
    [cVideoView PostInitialize];
    [cSettingsTableView PostInitialize];
}

/////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////

- (void) OnDialCallStarted :(NSString*) sAccountName :(int) nLineNo :(NSString*) sCallerName :(NSString*) sCallerId :(NSString*) sDialNo;
{
    [cVideoView PostDialCallStarted];
}

- (void) OnDialCallFailed :(NSString*) sAccountName :(int) nLineNo :(int) nStatusCode :(NSString*) sReasonPhrase :(NSString*) sContact;
{
    [cVideoView PostDialCallFailed :nStatusCode :sReasonPhrase];
}

/////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////

- (void) OnConnectedCall :(NSString*) sAccountName :(int) nLineNo;
{
    [[UIDevice currentDevice] setProximityMonitoringEnabled: YES];
    [cVideoView PostConnectedCall];
}

- (void) OnHungupCall :(NSString *)sAccountName :(int)nLineNo;
{
    [[UIDevice currentDevice] setProximityMonitoringEnabled: NO];
    [cVideoView PostDisconnectedCall];
}

- (void) OnDisconnectedCall :(int) nLineNo;
{
    [[UIDevice currentDevice] setProximityMonitoringEnabled: NO];
    [cVideoView PostDisconnectedCall];
}

/////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////

- (void) OnAddCallHistory :(NSString*) sAccountName :(Boolean) bOutboundCallType :(NSString*) sCallerName :(NSString*) sCallerId :(NSString*) sDialNo :(long) nStartTime :(long) nEndTime :(long) nDuration :(long) nDayNo :(int) nHistoryTypeId;
{
    NSMutableString* sResultName = [NSMutableString new];
    NSMutableString* sResultPhoneNo = [NSMutableString new];
    NSMutableString* sResultContactId = [NSMutableString new];
    
    if(bOutboundCallType)
    {
        [cVaxCallInfo PrepareInfo :sCallerName :sDialNo :&sResultName :&sResultPhoneNo :&sResultContactId];
    }
    else
    {
        [AppDelegate PostInboundCallHistory];
        [cVaxCallInfo PrepareInfo :sCallerName :sCallerId :&sResultName :&sResultPhoneNo :&sResultContactId];
    }
    
    cStoreRecents* pStoreDB = [cStoreRecents new];
    long nRecordId = [pStoreDB AddCallHistory :bOutboundCallType :sResultName :sResultPhoneNo :nStartTime :nEndTime :nDuration :nDayNo :nHistoryTypeId];

    [cRecentTableView PostAddCallHistory :nRecordId :bOutboundCallType :sResultName :sResultPhoneNo :nStartTime :nEndTime :nDuration :nDayNo :nHistoryTypeId];
}

/////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////

- (void) OnVoiceMailMsg :(NSString*) sAccountName :(Boolean) bIsMsgWaiting :(int) nNewMsgCount :(int) nOldMsgCount :(int) nNewUrgentMsgCount :(int) nOldUrgentMsgCount :(NSString*) sMsgAccount;
{
    
}


@end
