
//  Created by VaxSoft
//  Copyright © 2025 VaxSoft. All rights reserved.

#import <UIKit/UIAlertView.h>
#import "cVaxPhoneVideo.h"

#import "cVideoView.h"
#import "cSettingVideoQualityTableView.h"
#import "cSettingVideoBitrateTableView.h"

static cVaxPhoneVideo* mSelf = NULL;

@implementation cVaxPhoneVideo

- (id) init;
{
    self = [super init];
    mSelf = self;
    
    mVaxAgentSIP = [[cVaxUserAgentSIP alloc] initWith :self];
    
    m_bFrontVideoCameraCaptured = true;
    m_bVideoEnabled = true;
    m_bVideoDeviceCaptured = false;
    
    return self;
}

- (void) dealloc;
{
    mVaxAgentSIP = NULL;
}

+ (cVaxPhoneVideo*) GetSelf;
{
    return mSelf;
}

/////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////

- (bool) IsVideoEnabled;
{
    return m_bVideoEnabled;
}

- (bool) EnableVideo :(bool) bEnable;
{
    if(![mVaxAgentSIP EnableVideo :0 :bEnable :bEnable])
    {
        return false;
    }
    
    m_bVideoEnabled = bEnable;
    return true;
}

/////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////

- (void) ApplyVideoQuality;
{
    if(!m_bVideoDeviceCaptured)
        return;
    
    [self CloseVideoDevice];
    [self OpenVideoDevice];
}

- (bool) OpenVideoDevice;
{
    if(m_bVideoDeviceCaptured)
        return true;
    
    [self CloseVideoDevice];
    
    int nQuality = [cSettingVideoQualityTableView GetVideoQuality];
    
    int nDeviceId = 0;
    
    if(m_bFrontVideoCameraCaptured)
        nDeviceId = 0;
    else
        nDeviceId = 1;
    
    if(![mVaxAgentSIP OpenVideoDev :nDeviceId :nQuality])
    {
        return false;
    }
    
    m_bVideoDeviceCaptured = true;
    return true;
}

- (void) CloseVideoDevice;
{
    m_bVideoDeviceCaptured = false;
    [mVaxAgentSIP CloseVideoDev];
}

- (void) RotateVideoDevice;
{
    if(!m_bVideoDeviceCaptured)
        return;
    
    if(m_bFrontVideoCameraCaptured)
        m_bFrontVideoCameraCaptured = false;
    else
        m_bFrontVideoCameraCaptured = true;
    
    [self CloseVideoDevice];
    [self OpenVideoDevice];
}

- (void) ApplyVideoBitrate :(int) nQuality;
{
    [mVaxAgentSIP VideoCodecBitRate :VAX_VIDEO_CODEC_VP8 :nQuality];
}

/////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////

- (void) OnInitialized;
{
    [cSettingVideoBitrateTableView PostInitialize];
}

- (void) OnUnInitialized;
{
    [self CloseVideoDevice];
}

/////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////

- (void) OnVideoDeviceFrameRGB :(int) nDeviceId :(char*) pFrameRGB :(int) nFrameSize :(int) nFrameWidth :(int) nFrameHeight;
{
    [cVideoView PostDeviceFrameRGB :pFrameRGB :nFrameSize :nFrameWidth :nFrameHeight];
}

- (void) OnVideoRemoteFrameRGB :(NSString*) sAccountName :(int) nLineNo :(char*) pFrameRGB :(int) nFrameSize :(int) nFrameWidth :(int) nFrameHeight;
{
    [cVideoView PostRemoteFrameRGB :pFrameRGB :nFrameSize :nFrameWidth :nFrameHeight];
}

@end
