
//  Created by VaxSoft
//  Copyright © 2025 VaxSoft. All rights reserved.

#import <UIKit/UIAlertView.h>
#import "cVaxPhoneSIP.h"

#import "cSettingNetworkTableView.h"
#import "cAccountLoginView.h"

#import "cMsgAlert.h"
#import "cStoreAccount.h"

#define ACCOUNT_REGISTER_EXPIRE     1800

#define PUSH_TEAM_ID                @"B4NJYW6YVA"
#define PUSH_BUNDLE_ID              @"com.VaxSoft.VaxPhoneSDK.iOS"

static cVaxPhoneSIP* mSelf = NULL;

@implementation cVaxPhoneSIP

- (id) init;
{
    m_bOnline = false;
    m_bRegistrationSIP = false;
    
    self = [super init];
    mSelf = self;
    
    mVaxAgentSIP = [[cVaxUserAgentSIP alloc] initWith :self];
            
    mCallKit = [[cVaxCallKit alloc] initWith: self];
    
    mVaxPhoneGeneral = [cVaxPhoneGeneral new];
    mVaxPhoneCodec = [cVaxPhoneCodec new];
    mVaxPhoneDTMF = [cVaxPhoneDTMF new];
    mVaxPhoneRing = [cVaxPhoneRing new];
    mVaxPhoneLog = [cVaxPhoneLog new];
    mVaxPhoneVideo = [cVaxPhoneVideo new];
    mVaxPhoneChat = [cVaxPhoneChat new];
    mVaxPhoneStatus = [cVaxPhoneStatus new];
    mVaxPhoneMicSpk = [cVaxPhoneMicSpk new];
    mVaxPhoneVoiceChanger = [cVaxPhoneVoiceChanger new];
    
    mIncomingCallAlert = [[cIncomingCallAlert alloc] initWith :self];
    mBackForeNotify = [[cVaxBackForeNotify alloc] initWith: self];
    
    return self;
}

- (void) dealloc;
{
    mIncomingCallAlert = NULL;
    mBackForeNotify = NULL;
    
    mVaxPhoneGeneral = NULL;
    mVaxPhoneCodec = NULL;
    mVaxPhoneDTMF = NULL;
    mVaxPhoneRing = NULL;
    mVaxPhoneLog = NULL;
    mVaxPhoneVideo = NULL;
    mVaxPhoneChat = NULL;
    mVaxPhoneMicSpk = NULL;
    mVaxPhoneVoiceChanger = NULL;
    
    mVaxAgentSIP = NULL;
}

+ (cVaxPhoneSIP*) GetSelf;
{
    return mSelf;
}

/////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////

- (bool) Initialize;
{
    cStoreAccount* pStore = [cStoreAccount new];
    
    Boolean bRegistrationSIP = [pStore GetRegistration];
    
    NSString* sUserName    = [pStore GetUserName];
    NSString* sDisplayName = [pStore GetDisplayName];

    NSString* sAuthLogin   = [pStore GetAuthLogin];
    NSString* sAuthPwd     = [pStore GetAuthPwd];
    
    NSString* sDomainRealm = [pStore GetDomainRealm];
    
    NSString* sServerAddr  = [pStore GetServerAddr];
    NSString* sServerPort  = [pStore GetServerPort];
    
    ///////////////////////////////////////////////////////////////////////////////
    
    if([sAuthLogin length] == 0)
    {
        [cMsgAlert ShowAlert: @"Please enter Auth Login"];
        return false;
    }
    
    if([sServerAddr length] == 0)
    {
        [cMsgAlert ShowAlert: @"Please enter SIP Server address"];
        return false;
    }
    
    ///////////////////////////////////////////////////////////////////////////////
    
    [mVaxAgentSIP SetLicenceKey: @"TRIAL-LICENSE-KEY"];
    
    ///////////////////////////////////////////////////////////////////////////////
    
    int nLocalPortSIP = -1;
    
    if(![cSettingNetworkTableView IsRandomPortSIP])
        nLocalPortSIP = [cSettingNetworkTableView GetNetworkPortSIP];
    
    if(![mVaxAgentSIP Initialize])
        return false;

    //[mVaxAgentSIP SetNetworkPrioritySIP: VAX_NETWORK_MODE_PRIORITY_IPV4];
    
    [mVaxAgentSIP RemoveAccountSIP :ACCOUNT_VAXVOIP];

    if(![mVaxAgentSIP AddAccountSIP :ACCOUNT_VAXVOIP :VAX_ACCOUNT_TYPE_UDP :@"" :nLocalPortSIP :sDisplayName :sUserName :sAuthLogin :sAuthPwd :sDomainRealm :sServerAddr :[sServerPort intValue]])
        return false;
    
    m_bRegistrationSIP = bRegistrationSIP;
    
    if(bRegistrationSIP)
    {
        [mVaxAgentSIP RegisterAccountPushKit :ACCOUNT_VAXVOIP :ACCOUNT_REGISTER_EXPIRE];
    }
    
    [mVaxAgentSIP AutoRegistration :ACCOUNT_VAXVOIP :true :-1 :20];
    [mVaxAgentSIP EnableKeepAlive :ACCOUNT_VAXVOIP :10]; // 10 seconds keep alive;
    
    [mBackForeNotify BackForegroundNotify :true];
    [mVaxAgentSIP NetworkReachability :true];
    
    m_bOnline = true;
    return true;
}

- (void) UnInitialize;
{
    m_bOnline = false;
    m_bRegistrationSIP = false;
    
    [mVaxAgentSIP NetworkReachability :false];
    [mBackForeNotify BackForegroundNotify :false];
    
    [mVaxAgentSIP UnRegisterAccountSIP :ACCOUNT_VAXVOIP];
}

/////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////

- (Boolean) IsOnline;
{
    return m_bOnline;
}

/////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////

- (Boolean) DialCall :(NSString*) sDialNo;
{
    return [mVaxAgentSIP DialCall :ACCOUNT_VAXVOIP :0 :@"" :@"" :sDialNo];
}

- (Boolean) DisconnectCall;
{
    return [mVaxAgentSIP DisconnectCall :0];
}

/////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////

- (void) OnSuccessToRegister :(NSString*) sAccountName;
{
    [cAccountLoginView PostRegisterSuccessAccountSIP];
}

- (void) OnVaxRegisterSuccessPushKit :(NSString*) sDeviceToken;
{
    if(!m_bRegistrationSIP)
        return;
    
    [mVaxAgentSIP SetPushSIP :ACCOUNT_VAXVOIP :PUSH_TEAM_ID :PUSH_BUNDLE_ID :sDeviceToken];
    [mVaxAgentSIP RegisterAccountSIP :ACCOUNT_VAXVOIP :ACCOUNT_REGISTER_EXPIRE];
}

/////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////

- (void) OnIncomingCallPushKit :(NSString*) sCallerName :(NSString*) sCallerId;
{
    if(!m_bRegistrationSIP || [mVaxAgentSIP IsLineBusy :0])
    {
        return;
    }
    
    [mVaxAgentSIP RegisterAccountPushKit :ACCOUNT_VAXVOIP :ACCOUNT_REGISTER_EXPIRE];
    
    [mCallKit ShowUI :sCallerName :sCallerId];
}

/////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////

- (void) OnIncomingCallStarted :(NSString*) sAccountName :(NSString*) sCallerName :(NSString*) sCallerId;
{
    if([mVaxAgentSIP IsLineConnected :0])
    {
        [mVaxAgentSIP RejectCall];
        return;
    }
    
    #if TARGET_IPHONE_SIMULATOR
        [mIncomingCallAlert ShowAlert :sCallerName :sCallerId];
    #else
        [mCallKit ShowUI :sCallerName :sCallerId];
    #endif
}

- (void) OnIncomingCallEnded :(NSString*) sAccountName :(int) nReasonCode;
{
    #if TARGET_IPHONE_SIMULATOR
        [mIncomingCallAlert HideAlert];
    #else
        if(nReasonCode != VAX_INCOMING_CALL_ACCEPTED)
        {
            [mCallKit HideUI];
        }
    #endif
}

/////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////

- (void) OnVaxAnswerCallKit;
{
    [mVaxAgentSIP AcceptCall :0];
}

- (void) OnVaxEndCallKit;
{
    if([mVaxAgentSIP IsLineConnected :0])
    {
        [mVaxAgentSIP DisconnectCall :0];
        return;
    }
    
    [mVaxAgentSIP RejectCall];
}

/////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////

- (void) OnHungupCall :(NSString*) sAccountName :(int) nLineNo;
{
    [mCallKit HideUI];
}

- (void) OnDisconnectedCall :(int) nLineNo;
{
    [mCallKit HideUI];
}

/////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////

- (void) OnIncomingCallAlertAccept;
{
    [mVaxAgentSIP AcceptCall :0];
}

- (void) OnIncomingCallAlertReject;
{
    [mVaxAgentSIP RejectCall];
}

- (void) OnIncomingCallAlertMute;
{
    [mVaxPhoneRing MuteRingTone];
}

/////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////

- (void) OnNetworkReachability :(Boolean) bAvailable;
{
    if(bAvailable == false || [self IsOnline] == false)
    {
        return;
    }
    
    if([mVaxAgentSIP IsLineConnected :0])
    {
        [mVaxAgentSIP RefreshLine :0];
    }
}

/////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////

- (void) OnVaxApplicationEnterForeground;
{
    if(m_bRegistrationSIP)
      [mVaxAgentSIP RegisterAccountPushKit :ACCOUNT_VAXVOIP :ACCOUNT_REGISTER_EXPIRE];
}

/////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////

@end
