
//  Created by VaxSoft
//  Copyright © 2025 VaxSoft. All rights reserved.

#import "cAccountInfoTableView.h"
#import "cAccountInfoRowView.h"

#import "cCustomTextField.h"
#import "cCustomCellCheckBox.h"

#import "cStoreAccount.h"

#define SECTION_TOTAL_COUNT		2

#define SECTION_ID_PROFILE		0
#define SECTION_ID_SETTINGS     1

#define SECTION_PROFILE_ROWS    4
#define SECTION_SETTING_ROWS    4

#define ROW_ID_USER_NAME        0
#define ROW_ID_DISPLAY_NAME     1
#define ROW_ID_AUTH_LOGIN       2
#define ROW_ID_AUTH_PASSWORD    3

#define ROW_ID_DOMAIN_REALM     0
#define ROW_ID_SERVER_IP        1
#define ROW_ID_SERVER_PORT      2
#define ROW_ID_REGISTER_SIP     3

//////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////

static NSString* m_sAccountInfo_UserName = @"";
static NSString* m_sAccountInfo_DisplayName = @"";
static NSString* m_sAccountInfo_AuthLogin = @"";
static NSString* m_sAccountInfo_AuthPwd = @"";

static NSString* m_sAccountInfo_DomainRealm = @"";
static NSString* m_sAccountInfo_ServerAddr = @"";
static NSString* m_sAccountInfo_ServerPort = @"5060";

static Boolean m_bAccountInfo_Registration = true;

//////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////

@interface cAccountInfoTableView ()
@end

@implementation cAccountInfoTableView

- (void)viewDidLoad
{
    [super viewDidLoad];
}

- (void) didReceiveMemoryWarning
{
    [super didReceiveMemoryWarning];
}

//////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////

+ (void) PreloadAccountInfo :(NSString*) sUserName :(NSString*) sDisplayName :(NSString*) sAuthLogin :(NSString*) sAuthPwd :(NSString*) sDomainRealm :(NSString*) sServerAddr :(NSString*) sServerPort :(Boolean) bRegisterSIP;
{
    m_sAccountInfo_UserName = sUserName;
    m_sAccountInfo_DisplayName = sDisplayName;
    
    m_sAccountInfo_AuthLogin = sAuthLogin;
    m_sAccountInfo_AuthPwd = sAuthPwd;
    
    m_sAccountInfo_DomainRealm = sDomainRealm;
    m_sAccountInfo_ServerAddr = sServerAddr;
    m_sAccountInfo_ServerPort = sServerPort;
    
    m_bAccountInfo_Registration = bRegisterSIP;
}

//////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////

#pragma mark - Table view data source

- (NSInteger) numberOfSectionsInTableView:(UITableView *)tableView
{
    return SECTION_TOTAL_COUNT;
}

- (NSInteger) tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section
{
    if(section == SECTION_ID_PROFILE)
        return SECTION_PROFILE_ROWS;
    
    return SECTION_SETTING_ROWS;
}

- (NSString *)tableView:(UITableView *)tableView titleForHeaderInSection:(NSInteger)section
{
    if(section == SECTION_ID_PROFILE)
        return @"Account Profile";
    
    if(section == SECTION_ID_SETTINGS)
        return @"Account Settings";
    
    return @"";
}

//////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
    if(indexPath.row == ROW_ID_USER_NAME && indexPath.section == SECTION_ID_PROFILE)
    {
        cAccountInfoRowView* pRowView = [tableView dequeueReusableCellWithIdentifier:@"UserNameCellId" forIndexPath:indexPath];
        
        [pRowView SetTextField :m_sAccountInfo_UserName];
        return pRowView;
    }
    
    if(indexPath.row == ROW_ID_DISPLAY_NAME && indexPath.section == SECTION_ID_PROFILE)
    {
        cAccountInfoRowView* pRowView = [tableView dequeueReusableCellWithIdentifier:@"DisplayNameCellId" forIndexPath:indexPath];
        
        [pRowView SetTextField :m_sAccountInfo_DisplayName];
        return pRowView;
    }
    
    if(indexPath.row == ROW_ID_AUTH_LOGIN && indexPath.section == SECTION_ID_PROFILE)
    {
        cAccountInfoRowView* pRowView = [tableView dequeueReusableCellWithIdentifier:@"AuthLoginCellId" forIndexPath:indexPath];
        
        [pRowView SetTextField :m_sAccountInfo_AuthLogin];
        return pRowView;
    }
    
    if(indexPath.row == ROW_ID_AUTH_PASSWORD && indexPath.section == SECTION_ID_PROFILE)
    {
        cAccountInfoRowView* pRowView = [tableView dequeueReusableCellWithIdentifier:@"AuthPasswordCellId" forIndexPath:indexPath];
        
        [pRowView SetTextField :m_sAccountInfo_AuthPwd];
        return pRowView;
    }
    
    if(indexPath.row == ROW_ID_DOMAIN_REALM && indexPath.section == SECTION_ID_SETTINGS)
    {
        cAccountInfoRowView* pRowView = [tableView dequeueReusableCellWithIdentifier:@"DomainCellId" forIndexPath:indexPath];
        
        [pRowView SetTextField :m_sAccountInfo_DomainRealm];
        return pRowView;
    }
    
    if(indexPath.row == ROW_ID_SERVER_IP && indexPath.section == SECTION_ID_SETTINGS)
    {
        cAccountInfoRowView* pRowView = [tableView dequeueReusableCellWithIdentifier:@"ServerIPCellId" forIndexPath:indexPath];
        
        [pRowView SetTextField :m_sAccountInfo_ServerAddr];
        return pRowView;
    }
    
    if(indexPath.row == ROW_ID_SERVER_PORT && indexPath.section == SECTION_ID_SETTINGS)
    {
        cAccountInfoRowView* pRowView = [tableView dequeueReusableCellWithIdentifier:@"ServerPortCellId" forIndexPath:indexPath];
        
        [pRowView SetTextField :m_sAccountInfo_ServerPort];
        return pRowView;
    }

    cAccountInfoRowView* pRowView = [tableView dequeueReusableCellWithIdentifier:@"SIPRegisterCellId" forIndexPath:indexPath];
    
    [pRowView SetCheckBoxState :m_bAccountInfo_Registration];
    return pRowView;
}

//////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////

- (void) prepareForSegue:(UIStoryboardSegue *)segue sender:(id)sender
{
    if ([[segue identifier] isEqualToString :@"UnwindSave"])
    {
        [self StoreInfoAll];
    }
    
    else if ([[segue identifier] isEqualToString :@"UnwindCancel"])
    {
    }
}

- (void) StoreInfoAll;
{
    cStoreAccount* pStore = [cStoreAccount new];
    
    [pStore SetUserName :m_sAccountInfo_UserName];
    [pStore SetDisplayName :m_sAccountInfo_DisplayName];
    
    [pStore SetAuthLogin :m_sAccountInfo_AuthLogin];
    [pStore SetAuthPwd :m_sAccountInfo_AuthPwd];
    
    [pStore SetDomainRealm :m_sAccountInfo_DomainRealm];
    [pStore SetServerAddr :m_sAccountInfo_ServerAddr];
    [pStore SetServerPort :m_sAccountInfo_ServerPort];
    
    [pStore SetRegistration :m_bAccountInfo_Registration];
}

//////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////

- (IBAction) OnEditingUserName :(id)sender;
{
    cCustomTextField* pTextField = (cCustomTextField*) sender;
    m_sAccountInfo_UserName = [pTextField text];
}

- (IBAction) OnEditingDisplayName :(id)sender;
{
    cCustomTextField* pTextField = (cCustomTextField*) sender;
    m_sAccountInfo_DisplayName = [pTextField text];
}

- (IBAction) OnEditingAuthLogin :(id)sender;
{
    cCustomTextField* pTextField = (cCustomTextField*) sender;
    m_sAccountInfo_AuthLogin = [pTextField text];
}

- (IBAction) OnEditingAuthPwd :(id)sender;
{
    cCustomTextField* pTextField = (cCustomTextField*) sender;
    m_sAccountInfo_AuthPwd = [pTextField text];
}

- (IBAction) OnEditingDomainRealm :(id)sender;
{
    cCustomTextField* pTextField = (cCustomTextField*) sender;
    m_sAccountInfo_DomainRealm = [pTextField text];
}

- (IBAction) OnEditingServerAddr :(id)sender;
{
    cCustomTextField* pTextField = (cCustomTextField*) sender;
    m_sAccountInfo_ServerAddr = [pTextField text];
}

- (IBAction) OnEditingServerPort :(id)sender;
{
    cCustomTextField* pTextField = (cCustomTextField*) sender;
    m_sAccountInfo_ServerPort = [pTextField text];
}

- (IBAction) OnClickRegisterSIP :(id)sender;
{
    cCustomCellCheckBox* pCheckBox = (cCustomCellCheckBox*) sender;
    m_bAccountInfo_Registration = [pCheckBox GetState];
}

@end
