
//  Created by VaxSoft
//  Copyright © 2025 VaxSoft. All rights reserved.

#import "cAccountLoginView.h"
#import "cAccountInfoTableView.h"

#import "cVaxPhoneSIP.h"
#import "cStoreAccount.h"

#import "cCustomTextField.h"

#define DEFAULT_SIP_PORT    "5060"

static cAccountLoginView* mpAccountLoginTableView = NULL;
static NSMutableString* msAccountLoginStatusText = NULL;

//////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////

@interface cAccountLoginView ()
@end

@implementation cAccountLoginView

- (void) viewDidLoad
{
    m_bCheckAccountStatus = true;
    
    [super viewDidLoad];
    
    [mAuthLogin EnableKeyboardReturnKey];
    [mAuthPassword EnableKeyboardReturnKey];
    [mServerURI EnableKeyboardReturnKey];
    
    msAccountLoginStatusText = [NSMutableString new];
    [msAccountLoginStatusText setString :@"Account is offline"];
    
    mScrollView.contentSize = mScrollView.frame.size;
}

- (void) didReceiveMemoryWarning
{
    [super didReceiveMemoryWarning];
}

//////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////

- (void) viewDidAppear: (BOOL) animated
{
    [super viewDidAppear :animated];
}

- (void) viewWillAppear: (BOOL) animated
{
    mpAccountLoginTableView = self;
    
    [self DisplayLoginInfo];
    
    [self ShowStatusMsg :msAccountLoginStatusText];
    [self UpdateOnlineBtnText];
    
    [super viewWillAppear :animated];
    
    if(m_bCheckAccountStatus)
    {
        m_bCheckAccountStatus = false;
        cStoreAccount* pStore = [cStoreAccount new];
    
        if([pStore GetAccountStatusOnline])
        {
            [self AccountOnline];
        }
    }
}

- (void) viewWillDisappear: (BOOL)animated;
{
    mpAccountLoginTableView = NULL;
    [super viewWillDisappear :animated];
}

//////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////

+ (void) PostStatusText :(NSString*) sMsgText;
{
    if(msAccountLoginStatusText != NULL)
        [msAccountLoginStatusText setString :sMsgText];
    
    if(mpAccountLoginTableView != NULL)
        [mpAccountLoginTableView ShowStatusMsg :sMsgText];
}

+ (void) PostRegisterSuccessAccountSIP;
{
    if(mpAccountLoginTableView != NULL)
    {
        cStoreAccount* pStore = [cStoreAccount new];
        [pStore SetAccountStatusOnline: true];
        
        [mpAccountLoginTableView ShowMainTabBar :true];
    }
}

//////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////

- (IBAction) OnClickSignin:(id)sender;
{
    if([cVaxPhoneSIP.GetSelf IsOnline])
    {
        [self AccountOffline];
        return;
    }
    
    [self AccountOnline];
}

- (void) AccountOffline;
{
    [self ShowMainTabBar :false];
    
    [[cVaxPhoneSIP GetSelf] UnInitialize];
    [self UpdateOnlineBtnText];
    
    cStoreAccount* pStore = [cStoreAccount new];
    [pStore SetAccountStatusOnline: false];
}

- (void) AccountOnline;
{
    if(![cVaxPhoneGeneral.GetSelf IsNetworkAvailable])
    {
        [cAccountLoginView PostStatusText :@"Network is not available"];
        return;
    }
    
    [self StoreLoginInfo];
    
    Boolean bRegisterSIP = mRegisterSIP.on;
    
    if(![[cVaxPhoneSIP GetSelf] Initialize])
    {
        return;
    }
    
    [self UpdateOnlineBtnText];
    
    if(bRegisterSIP == false)
    {
        cStoreAccount* pStore = [cStoreAccount new];
        [pStore SetAccountStatusOnline: true];
        
        [self ShowMainTabBar :true];
    }
}

//////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////

- (IBAction) OnClickTestAccount:(id)sender;
{
    NSTimeInterval nDateTime = [[NSDate date] timeIntervalSince1970];
    NSString* sDateTime = [NSString stringWithFormat:@"%ld", ((long) nDateTime) % 9999999];
    
    [mAuthLogin setText :sDateTime];
    [mAuthPassword setText :sDateTime];
    
    [mServerURI setText :@"demo.vaxvoip.com:8060"];
    [mRegisterSIP setOn: true];
    
    cStoreAccount* pStore = [cStoreAccount new];
    
    [pStore SetUserName: sDateTime];
    [pStore SetDisplayName: sDateTime];
    [pStore SetDomainRealm: @"demo.vaxvoip.com"];
    
    [self ShowTestAccountInfo :[mAuthLogin text] :[mAuthPassword text] :@"demo.vaxvoip.com" :@"8060"];
}

//////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////

- (void) ShowStatusMsg :(NSString*) sMsgText;
{
    [mStatusText setText :sMsgText];
}

//////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////

- (IBAction) OnEditingLoginAuth:(id)sender;
{
    cStoreAccount* pStore = [cStoreAccount new];
    
    [pStore SetUserName: @""];
    [pStore SetDisplayName: @""];
    [pStore SetDomainRealm: @""];
}

- (IBAction) OnEditingServerAddr:(id)sender;
{
    cStoreAccount* pStore = [cStoreAccount new];
    
    [pStore SetUserName: @""];
    [pStore SetDisplayName: @""];
    [pStore SetDomainRealm: @""];
}

//////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////

- (BOOL) prefersStatusBarHidden
{
    return NO;
}

- (void) OnShowMainTabBar;
{
    [self performSegueWithIdentifier: @"SegueMainTabBar" sender: self];
}

- (void) ShowMainTabBar :(Boolean) bStart;
{
    [NSObject cancelPreviousPerformRequestsWithTarget:self selector:@selector(OnShowMainTabBar) object:nil];
    
    if(bStart == true)
        [self performSelector :@selector(OnShowMainTabBar) withObject:nil afterDelay:1.0];
}

//////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////

- (void) UpdateOnlineBtnText;
{
    if([[cVaxPhoneSIP GetSelf] IsOnline])
    {
        [mOnlineBtn setTitle :@"Sign out" forState:UIControlStateNormal];
    }
    else
    {
        [mOnlineBtn setTitle :@"Sign in" forState:UIControlStateNormal];
    }
}

//////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////

- (IBAction) UnwindFromAccountInfoTableView:(UIStoryboardSegue *)segue;
{
}

- (void) prepareForSegue:(UIStoryboardSegue *)segue sender:(id)sender
{
    if ([[segue identifier] isEqualToString:@"SegueAccountInfo"])
    {
        cStoreAccount* pStore = [cStoreAccount new];
        
        NSString* sUserName = [pStore GetUserName];
        if(sUserName == nil || [sUserName length] == 0) sUserName = mAuthLogin.text;
        
        NSString* sDisplayName = [pStore GetDisplayName];
        if(sDisplayName == nil || [sDisplayName length] == 0) sDisplayName = mAuthLogin.text;
        
        //////////////////////////////////////////////////////////////////////////////////////////////////
        
        NSMutableString* sServerAddr = [NSMutableString new];
        NSMutableString* sServerPort = [NSMutableString new];
        
        [self ParseServerURI :mServerURI.text :&sServerAddr :&sServerPort];
        
        //////////////////////////////////////////////////////////////////////////////////////////////////
        
        NSString* sDomainRealm = [pStore GetDomainRealm];
        if(sDomainRealm == nil || [sDomainRealm length] == 0) sDomainRealm = sServerAddr;
        
        [cAccountInfoTableView PreloadAccountInfo :sUserName :sDisplayName :mAuthLogin.text :mAuthPassword.text :sDomainRealm :sServerAddr :sServerPort :mRegisterSIP.on];
    }
}

//////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////

- (void) ShowTestAccountInfo :(NSString*) sAuthLogin :(NSString*) sAuthPwd :(NSString*) sServerAddr :(NSString*) sServerPort;
{
    NSString* sMsgText = [NSString stringWithFormat:@"Auth Login: %@\nAuth Password: %@\nServer Addr: %@\nServer Port: %@\n\nDemo accounts are only for account-to-account VoIP calls. Use demo accounts in different devices to dial/receive VoIP calls.\n\nDial 000 for self testing.\n\n[Powered by VaxVoIP Server SDK]", sAuthLogin, sAuthPwd, sServerAddr, sServerPort];
    
    UIAlertController* MsgAlert = [UIAlertController alertControllerWithTitle:@"VaxPhone" message:sMsgText preferredStyle:UIAlertControllerStyleAlert];
    
    UIAlertAction* OkAction = [UIAlertAction actionWithTitle:@"OK" style:UIAlertActionStyleDefault handler:^(UIAlertAction * action)
                               {
                                   
                               }];
    
    [MsgAlert addAction:OkAction];
    
    [self presentViewController :MsgAlert animated:YES completion:nil];
}

/////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////

- (NSString*) GenServerURI;
{
    cStoreAccount* pStore = [cStoreAccount new];
    
    NSString* sServerAddr = [pStore GetServerAddr];
    if(sServerAddr.length == 0) return @"";
    
    NSString* sServerPort = [pStore GetServerPort];
    
    NSRange rangeValue = [sServerAddr rangeOfString:@":" options:NSCaseInsensitiveSearch];
    
    if(rangeValue.length > 0)
    {
        //Server Address is IPv6
        
        return [NSString stringWithFormat :@"[%@]:%@", sServerAddr, sServerPort];
    }
    
    return [NSString stringWithFormat :@"%@:%@", sServerAddr, sServerPort];
}

- (void) DisplayLoginInfo;
{
    cStoreAccount* pStore = [cStoreAccount new];
    
    [mAuthLogin setText :[pStore GetAuthLogin]];
    [mAuthPassword setText :[pStore GetAuthPwd]];
    
    [mServerURI setText :[self GenServerURI]];
    mRegisterSIP.on = [pStore GetRegistration];
}

/////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////

- (void) ParseServerURI :(NSString*) sURI :(NSMutableString**) sResultServerAddr :(NSMutableString**) sResultServerPort;
{
    if(sURI == nil || sURI.length == 0) return;
    
    NSArray* aCompURI = [sURI componentsSeparatedByString:@":"];
    
    if(aCompURI.count == 1)
    {
        //URI is IPv4 without Port
        
        [*sResultServerAddr setString :[aCompURI objectAtIndex: 0]];
        [*sResultServerPort setString :@DEFAULT_SIP_PORT];
        
        return;
    }
    
    if(aCompURI.count == 2)
    {
        //URI is IPv4 with Port
        
        [*sResultServerAddr setString :[aCompURI objectAtIndex: 0]];
        [*sResultServerPort setString :[aCompURI objectAtIndex: 1]];
        
        return;
    }
    
    aCompURI = [sURI componentsSeparatedByString:@"]:"];
    
    if(aCompURI.count == 1)
    {
        // URI is IPv6 without port
        
        NSString* sServerAddr = [aCompURI objectAtIndex: 0];
        
        sServerAddr = [sServerAddr stringByReplacingOccurrencesOfString :@"[" withString:@""];
        sServerAddr = [sServerAddr stringByReplacingOccurrencesOfString :@"]" withString:@""];
        
        [*sResultServerAddr setString :sServerAddr];
        [*sResultServerPort setString :@DEFAULT_SIP_PORT];
        
        return;
    }
    
    NSString* sServerAddr = [aCompURI objectAtIndex: 0];
    sServerAddr = [sServerAddr stringByReplacingOccurrencesOfString :@"[" withString:@""];
    
    [*sResultServerAddr setString :sServerAddr];
    [*sResultServerPort setString :[aCompURI objectAtIndex: 1]];
}

- (void) StoreLoginInfo;
{
    NSString* sAuthLogin = [mAuthLogin text];
    NSString* sAuthPwd = [mAuthPassword text];
    
    NSString* sServerURI = [mServerURI text];
    Boolean bRegister = mRegisterSIP.on;
    
    ///////////////////////////////////////////////////////////////////////////////
    
    NSMutableString* sServerAddr = [NSMutableString new];
    NSMutableString* sServerPort = [NSMutableString new];
    
    [self ParseServerURI :sServerURI :&sServerAddr :&sServerPort];
    
    ///////////////////////////////////////////////////////////////////////////////
    
    cStoreAccount* pStore = [cStoreAccount new];
    
    [pStore SetAuthLogin :sAuthLogin];
    [pStore SetAuthPwd :sAuthPwd];
    
    [pStore SetServerAddr :sServerAddr];
    [pStore SetServerPort :sServerPort];
    
    [pStore SetRegistration :bRegister];
}

//////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////

@end
