
//  Created by VaxSoft
//  Copyright © 2025 VaxSoft. All rights reserved.

#import "cVaxPhoneSIP.h"

#import "cChatContactTableView.h"
#import "cChatMyStatusTableView.h"

#import "cChatContactRowData.h"
#import "cChatContactRowView.h"

#import "cChatMsgTableView.h"
#import "cChatMsgContainerView.h"
#import "cDialPadView.h"

#import "cMainTabBarController.h"
#import "cStoreChatContacts.h"

#define SECTION_TOTAL_COUNT      1
#define SECTION_ID_CHAT_CONTACT  0

static cChatContactTableView* mpChatContactTableView = nil;
static NSMutableArray* mpChatContactTableData = nil;
static int m_nChatContactMissedCount = 0;

@interface cChatContactTableView ()
@end

@implementation cChatContactTableView

- (void) viewDidLoad
{
    [super viewDidLoad];
}

- (void) didReceiveMemoryWarning
{
    [super didReceiveMemoryWarning];
}

+ (void) OnStartedAPP;
{
    mpChatContactTableData = [NSMutableArray new];
}

/////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////

- (void) viewWillAppear: (BOOL) animated
{
    mpChatContactTableView = self;
    
    [cChatContactTableView SetMissedCount :0];
    [cChatContactTableView ResetContactSelectedStateAll];
    
    [[cVaxPhoneChat GetSelf] ChatSubscribeContactAll];
    
    [self.tableView reloadData];
    [super viewWillAppear :animated];
}

- (void) viewWillDisappear: (BOOL)animated;
{
    mpChatContactTableView = nil;
    [super viewWillDisappear :animated];
}

//////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////

+ (void) ResetContactSelectedStateAll;
{
    for(int nRowNo = 0; nRowNo < [mpChatContactTableData count]; nRowNo++)
    {
        cChatContactRowData* pRowData = [mpChatContactTableData objectAtIndex :nRowNo];
        [pRowData SetContactSelected :NO];
    }
}

+ (void) PostInitialize;
{
    [[cVaxPhoneChat GetSelf] LoadChatContactAll];
}

//////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////

+ (void) PostChatContactAdded :(NSString*) sContactName :(long) nMissedCount :(Boolean) bPresence;
{
    cChatContactRowData* pRowData = [cChatContactTableView FindChatContact :sContactName];
    
    Boolean bAdded = false;
    
    if(pRowData == nil)
    {
        pRowData = [cChatContactRowData new];
        [pRowData SetContactName :sContactName];
        
        [mpChatContactTableData addObject :pRowData];
        bAdded = true;
    }
    
    [pRowData SetMissedCount :nMissedCount];
    [pRowData SetPresence :bPresence];
    
    if(mpChatContactTableView != nil && bAdded == true)
    {
        long nRowNo = [mpChatContactTableData count] - 1;
        [mpChatContactTableView AddRowView :nRowNo];
    }
}

- (void) AddRowView :(long) nRowNo;
{
    NSIndexPath* indexPath = [NSIndexPath indexPathForRow:nRowNo inSection:SECTION_ID_CHAT_CONTACT];
    [self.tableView insertRowsAtIndexPaths:@[indexPath] withRowAnimation:UITableViewRowAnimationAutomatic];
}

//////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////

+ (void) PostChatMessageText :(NSString*) sContactName :(Boolean) bIsChatContact :(NSString*) sMsgText :(Boolean) bMsgTypeOutgoing;
{
    if(bMsgTypeOutgoing == false && [cMainTabBarController IsChatContactSelected] == false)
        [cChatContactTableView SetMissedCount :-1];
    
    /////////////////////////////////////////////////////////////////////////
    
    cChatContactRowData* pRowData = [cChatContactTableView FindChatContact :sContactName];
    
    if(bMsgTypeOutgoing == false && pRowData != nil)
    {
        if([pRowData IsContactSelected] == false)
            [pRowData SetMissedCount :-1];
    }
    
    if([pRowData IsContactSelected] == true)
    {
        [cChatMsgTableView PostChatMessageText :sContactName :sMsgText :bMsgTypeOutgoing];
    }
    
    /////////////////////////////////////////////////////////////////////////
    
    if(bMsgTypeOutgoing == false && mpChatContactTableView != nil)
        [mpChatContactTableView UpdateMissedCountRowView :sContactName];
}

//////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////

+ (void) PostChatContactStatus :(NSString*) sContactName :(long) nStatusId;
{
    cChatContactRowData* pRowData = [cChatContactTableView FindChatContact :sContactName];
    
    if(pRowData != nil)
        [pRowData SetStatusId :nStatusId];
    
    if(mpChatContactTableView == nil)
        return;
    
    [mpChatContactTableView UpdateStatusRowView :sContactName :nStatusId];
}

+ (cChatContactRowData*) FindChatContact :(NSString*) sContactName;
{
    cChatContactRowData* pResultRowData = nil;
    
    for(int nRowNo = 0; nRowNo < [mpChatContactTableData count]; nRowNo++)
    {
        cChatContactRowData* pRowData = [mpChatContactTableData objectAtIndex :nRowNo];
        if(pRowData == nil) continue;
        
        NSString* sName = [pRowData GetContactName];
        
        if([sName compare: sContactName options:NSCaseInsensitiveSearch] == NSOrderedSame)
        {
            pResultRowData = pRowData;
            break;
        }
    }
    
    return pResultRowData;
}

//////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////

- (void) UpdateStatusRowView :(NSString*) sContactName :(long) nStatusId;
{
    for(int nRowNo = 0; nRowNo < [mpChatContactTableData count]; nRowNo++)
    {
        NSIndexPath* indexPath = [NSIndexPath indexPathForRow: nRowNo inSection: SECTION_ID_CHAT_CONTACT];
        if(indexPath == nil) continue;
    
        cChatContactRowView* pRowView = (cChatContactRowView*) [self.tableView cellForRowAtIndexPath: indexPath];
        if(pRowView == nil) continue;
        
        NSString* sName = [pRowView GetContactName];
        
        if([sName compare: sContactName options:NSCaseInsensitiveSearch] != NSOrderedSame)
            continue;
        
        [pRowView UpdateContactStatus];
    }
}

- (void) UpdateMissedCountRowView :(NSString*) sContactName;
{
    for(int nRowNo = 0; nRowNo < [mpChatContactTableData count]; nRowNo++)
    {
        NSIndexPath* indexPath = [NSIndexPath indexPathForRow: nRowNo inSection: SECTION_ID_CHAT_CONTACT];
        if(indexPath == nil) continue;
        
        cChatContactRowView* pRowView = (cChatContactRowView*) [self.tableView cellForRowAtIndexPath: indexPath];
        if(pRowView == nil) continue;
        
        NSString* sName = [pRowView GetContactName];
        
        if([sName compare: sContactName options:NSCaseInsensitiveSearch] != NSOrderedSame)
            continue;
        
        [pRowView UpdateContactMissedCount];
    }
}

/////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////

- (void) DeleteRow :(long) nRowNo;
{
    if(nRowNo >= [mpChatContactTableData count])
        return;
    
    cChatContactRowData* pRowData = [mpChatContactTableData objectAtIndex :nRowNo];
    
    [[cVaxPhoneChat GetSelf] ChatRemoveContact :[pRowData GetContactName]];
    [mpChatContactTableData removeObjectAtIndex :nRowNo];
    
    NSIndexPath* indexPath = [NSIndexPath indexPathForRow :nRowNo inSection :SECTION_ID_CHAT_CONTACT];
    [self.tableView deleteRowsAtIndexPaths :@[indexPath] withRowAnimation:UITableViewRowAnimationFade];
}

//////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////

#pragma mark - Table view data source

- (NSInteger) numberOfSectionsInTableView:(UITableView *)tableView
{
    return SECTION_TOTAL_COUNT;
}

- (NSInteger) tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section
{
    return [mpChatContactTableData count];
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
    cChatContactRowData* pRowData = [mpChatContactTableData objectAtIndex :indexPath.row];
    cChatContactRowView* pRowView = [tableView dequeueReusableCellWithIdentifier :@"ChatContactCellId" forIndexPath:indexPath];
    
    [pRowView SetData: pRowData];
    return pRowView;
}

- (void) prepareForSegue :(UIStoryboardSegue *)segue sender:(id)sender
{
    if ([[segue identifier] isEqualToString:@"SegueChatMsgContainerView"])
    {
        NSString* sAddContactMsg = @"";
        NSString* sContactName = @"";
        
        cChatContactRowData* pRowData = nil;
        
        if([sender isKindOfClass :[cChatContactRowData class]])
        {
            pRowData = (cChatContactRowData*) sender;
            
            sContactName = [pRowData GetContactName];
            sAddContactMsg = @"Hi, I'd like to add you as a chat contact.";
        }
        else
        {
            NSIndexPath* indexPath = [self.tableView indexPathForSelectedRow];
            if(indexPath == nil) return;
            
            pRowData = [mpChatContactTableData objectAtIndex :indexPath.row];
            
            sContactName = [pRowData GetContactName];
            sAddContactMsg = @"";
        }
        
        [pRowData SetMissedCount :0];
        [pRowData SetContactSelected :YES];
        
        [cChatMsgContainerView PostMessageViewData :sContactName :[pRowData GetPresence] :sAddContactMsg];
    }
}

- (Boolean) ShowAddressBookUI :(UILabel*) pTitleBar;
{
    mpAddressBookView = [[cAddressBookView alloc] initWith :self];
    [mpAddressBookView ShowContacts :self :true];
    
    return true;
}

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

- (IBAction) OnClickAddContact :(id)sender
{
    UIAlertController* MsgAlert = [UIAlertController alertControllerWithTitle:@"Chat Contact SIP" message:@"" preferredStyle:UIAlertControllerStyleAlert];
    
    UIAlertAction* AddAction = [UIAlertAction actionWithTitle:@"Add Account SIP" style:UIAlertActionStyleDefault handler:^(UIAlertAction * action)
                                {
                                    UITextField *txtAccountSIP = MsgAlert.textFields.firstObject;
                                    
                                    if(txtAccountSIP.text.length > 0)
                                        [self ChatContactAdd :txtAccountSIP.text];
                                    
                                    [MsgAlert dismissViewControllerAnimated:false completion:nil];
                                }];
    
    UIAlertAction* ChooseAction = [UIAlertAction actionWithTitle:@"Choose from contacts" style:UIAlertActionStyleDefault handler:^(UIAlertAction * action)
                                {
                                    [self ShowAddressBookUI: self->mTitleBarAB];
                                       
                                    [MsgAlert dismissViewControllerAnimated:false completion:nil];
                                }];
    
    UIAlertAction* CancelAction = [UIAlertAction actionWithTitle:@"Cancel" style:UIAlertActionStyleDefault handler:^(UIAlertAction * action)
                                {
                                       
                                }];
    
    [AddAction setValue:[UIColor darkGrayColor] forKey:@"titleTextColor"];
    [ChooseAction setValue:[UIColor darkGrayColor] forKey:@"titleTextColor"];
    [CancelAction setValue:[UIColor blackColor] forKey:@"titleTextColor"];
    
    [MsgAlert addTextFieldWithConfigurationHandler:^(UITextField* txtAccountSIP)
     {
         txtAccountSIP.placeholder = @"Account SIP";
     }];
    
    [MsgAlert addAction:AddAction];
    [MsgAlert addAction:ChooseAction];
    [MsgAlert addAction:CancelAction];
    
    [self presentViewController :MsgAlert animated:YES completion:nil];
}

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

- (void) ChatContactAdd :(NSString*) sContactTitle;
{
    if(![[cVaxPhoneChat GetSelf] ChatAddContact :sContactTitle :YES])
        return;
    
    cChatContactRowData* pRowData = [mpChatContactTableData objectAtIndex :[mpChatContactTableData count] - 1];
    [self performSegueWithIdentifier: @"SegueChatMsgContainerView" sender: pRowData];
}

- (void) OnAddressBookSelectedContact :(NSString*) sContactId :(NSString*) sContactTitle;
{
    [self ChatContactAdd :sContactTitle];
}

- (IBAction) UnwindFromChatMyStatusView :(UIStoryboardSegue *)segue;
{
}

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

- (BOOL) tableView:(UITableView *)tableView canEditRowAtIndexPath:(NSIndexPath *)indexPath
{
    return YES;
}

- (void)tableView:(UITableView *)tableView commitEditingStyle:(UITableViewCellEditingStyle)editingStyle forRowAtIndexPath:(NSIndexPath *)indexPath
{
    if (editingStyle == UITableViewCellEditingStyleDelete)
    {
        [self DeleteRow :indexPath.row];
    }
}

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

+ (void) LoadMissedCount;
{
    m_nChatContactMissedCount = [cChatContactTableView ReadMissedCount];
    [cMainTabBarController SetChatContactBadgeCount :m_nChatContactMissedCount];
}

+ (void) SetMissedCount :(int) nValue;
{
    if(nValue == 0)
    {
        m_nChatContactMissedCount = 0;
    }
    else
    {
        m_nChatContactMissedCount++;
    }
    
    [cMainTabBarController SetChatContactBadgeCount :m_nChatContactMissedCount];
    [cChatContactTableView StoreMissedCount];
}

+ (void) StoreMissedCount;
{
    NSUserDefaults* pRefs = [NSUserDefaults standardUserDefaults];
    
    [pRefs setObject :[NSNumber numberWithInt: m_nChatContactMissedCount] forKey:@"m_nChatContactMissedCount"];
    [pRefs synchronize];
}

+ (int) ReadMissedCount;
{
    NSUserDefaults* pRefs = [NSUserDefaults standardUserDefaults];
    
    NSNumber* pValue = [pRefs objectForKey :@"m_nChatContactMissedCount"];
    if(pValue == nil) return 0;
    
    return [pValue intValue];
}


@end
