
//  Created by VaxSoft
//  Copyright © 2025 VaxSoft. All rights reserved.

#import "cVaxPhoneSIP.h"

#import "cChatMsgTableView.h"
#import "cChatMsgRowData.h"
#import "cChatMsgRowView.h"

#import "cStoreChatMsgs.h"

#define SECTION_TOTAL_COUNT     1
#define SECTION_ID_CHAT_MSG     0

static cChatMsgTableView* m_pChatMsgTableView;
static NSMutableString* m_sChatMsgTableContactName = nil;
static Boolean m_bChatMsgTablePresence = false;

@interface cChatMsgTableView()
@end

@implementation cChatMsgTableView

- (void) viewDidLoad
{
    [super viewDidLoad];
    
    m_bDidAppear = false;
    
    self.tableView.rowHeight = UITableViewAutomaticDimension;
    self.tableView.estimatedRowHeight = 140.0;
    
    m_CellHeights = [NSMutableDictionary new];
}

- (void) didReceiveMemoryWarning
{
    [super didReceiveMemoryWarning];
}

//////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////

- (void) LoadChatMessageAll;
{
    cStoreChatMsgs* pStoreDB = [cStoreChatMsgs new];
    NSArray* pMessageAll = [pStoreDB GetChatMessageAll :m_sChatMsgTableContactName];
    
    for(long nRecordNo = 0; nRecordNo < [pMessageAll count]; nRecordNo++)
    {
        cStoreDataChatMsgs* pDataDB = [pMessageAll objectAtIndex :nRecordNo];
        [self AddChatMessageText :pDataDB->m_sContactName :pDataDB->m_sMsgText :pDataDB->m_bMsgTypeOutgoing];
    }
}

//////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////

+ (void) PostMessageViewData :(NSString*) sContactName :(Boolean) bPresence;
{
    m_sChatMsgTableContactName = [NSMutableString new];
    [m_sChatMsgTableContactName setString: sContactName];
    
    m_bChatMsgTablePresence = bPresence;
}

+ (void) PostChatMessageText :(NSString*) sContactName :(NSString*) sMsgText :(Boolean) bMsgTypeOutgoing;
{
    if(m_pChatMsgTableView == nil)
        return;
    
    if([sContactName compare: m_sChatMsgTableContactName options:NSCaseInsensitiveSearch] != NSOrderedSame)
        return;
    
    [m_pChatMsgTableView AddChatMessageText :sContactName :sMsgText :bMsgTypeOutgoing];
}

//////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////

- (void)viewDidLayoutSubviews
{
    [super viewDidLayoutSubviews];
    
    if(m_bDidAppear == false)
        [self ScrollTableViewToBottom :false];
}

//////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////

- (void) viewWillAppear :(BOOL)animated
{
    m_pChatMsgTableView = self;
    
    m_bDidAppear = false;
    mpTableData = [NSMutableArray new];
    
    [self LoadChatMessageAll];

    if(m_bChatMsgTablePresence)
    {
        self.tableView.tableHeaderView = nil;
    }
    else
    {
        self.tableView.tableHeaderView = mNewChatAcceptView;
    }
    
    [self.tableView reloadData];
    [super viewWillAppear :animated];
}

- (void) viewDidAppear :(BOOL)animated
{
    m_bDidAppear = true;
    [super viewDidAppear :animated];
}

/////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////

- (void)viewWillDisappear :(BOOL)animated
{
    m_pChatMsgTableView = nil;
    m_sChatMsgTableContactName = nil;
    
    [mpTableData removeAllObjects];
    mpTableData = nil;
    
    [super viewWillDisappear :animated];
}

//////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////

- (void) AddChatMessageText :(NSString*) sContactName :(NSString*) sMsgText :(Boolean) bMsgTypeOutgoing;
{
    cChatMsgRowData* pRowData = [cChatMsgRowData new];
    [pRowData SetChatMsg :sMsgText :bMsgTypeOutgoing];
    
    [mpTableData addObject :pRowData];
    
    if(m_bDidAppear)
    {
        long nRowNo = [mpTableData count] - 1;
        [self AddRowView :nRowNo];
    }
}

- (void) AddRowView :(long) nRowNo;
{
    NSIndexPath* indexPath = [NSIndexPath indexPathForRow:nRowNo inSection:SECTION_ID_CHAT_MSG];
    [self.tableView insertRowsAtIndexPaths:@[indexPath] withRowAnimation:UITableViewRowAnimationAutomatic];
        
    [self ScrollTableViewToBottom :true];
}

//////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////

+ (void) PostScrollToBottom;
{
    if(m_pChatMsgTableView == nil)
        return;
    
    [m_pChatMsgTableView ScrollTableViewToBottom :true];
}

- (void) ScrollTableViewToBottom :(Boolean) bAnimate;
{
    long nTableRowCount = [self.tableView numberOfRowsInSection :SECTION_ID_CHAT_MSG];
    if(nTableRowCount == 0) return;
    
    [self.tableView scrollToRowAtIndexPath:[NSIndexPath indexPathForRow:nTableRowCount - 1 inSection :SECTION_ID_CHAT_MSG] atScrollPosition:UITableViewScrollPositionBottom animated: bAnimate];
}
//////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////

#pragma mark - Table view data source

- (NSInteger) numberOfSectionsInTableView:(UITableView *)tableView
{
    return SECTION_TOTAL_COUNT;
}

- (NSInteger) tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section
{
    return [mpTableData count];
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
    cChatMsgRowData* pRowData = [mpTableData objectAtIndex :indexPath.row];
    cChatMsgRowView* pRowView = NULL;
    
    if([pRowData IsChatMsgTypeOutgoing])
        pRowView = [tableView dequeueReusableCellWithIdentifier:@"ChatOutgoingMsgCellId" forIndexPath:indexPath];
    else
        pRowView = [tableView dequeueReusableCellWithIdentifier:@"ChatIncomingMsgCellId" forIndexPath:indexPath];
    
    [pRowView SetData :pRowData];
    return pRowView;
}

//////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////

- (CGFloat)tableView:(UITableView *)tableView estimatedHeightForRowAtIndexPath:(NSIndexPath *)indexPath
{
    NSNumber* nHeight = [m_CellHeights objectForKey:indexPath];
    if(nHeight != NULL) return nHeight.doubleValue;
    
    return UITableViewAutomaticDimension;
}

- (void)tableView:(UITableView *)tableView willDisplayCell:(UITableViewCell *)cell forRowAtIndexPath:(NSIndexPath *)indexPath
{
    [m_CellHeights setObject:@(cell.frame.size.height) forKey:indexPath];
}

//////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////

- (void) ClearMsgAll;
{
    if(m_pChatMsgTableView == nil)
        return;
    
    cStoreChatMsgs* pStoreDB = [cStoreChatMsgs new];
    [pStoreDB RemoveMessageAll :m_sChatMsgTableContactName];
    
    [mpTableData removeAllObjects];
    
    [self.tableView reloadData];
    [m_CellHeights removeAllObjects];
}

+ (void) PostClearMsgAll;
{
    if(m_pChatMsgTableView == nil)
        return;
    
    [m_pChatMsgTableView ClearMsgAll];
}

- (IBAction) OnClickAcceptBtn :(id)sender
{
    if(![[cVaxPhoneChat GetSelf] ChatAddContact :m_sChatMsgTableContactName :YES])
        return;
    
    self.tableView.tableHeaderView = nil;
}

@end
