
//  Created by VaxSoft
//  Copyright © 2025 VaxSoft. All rights reserved.

#import "cIncomingCallAlert.h"
#import "cMainTabBarController.h"

#import "cVaxPhoneSIP.h"

#import "cVideoView.h"
#import "cMainTabBarController.h"

@implementation cIncomingCallAlert
@synthesize mCallAlertDelegate;


- (id)initWith :(id) iDelegate;
{
    mIncomingCallAlert = NULL;
    mCallAlertDelegate = iDelegate;
    
    self = [super init];
    return self;
}

- (void) dealloc;
{
    [self HideAlert];
}

- (NSString*) PrepareTitle :(NSString*) sCallerName :(NSString*) sCallerId;
{
    if([sCallerName compare: sCallerId options:NSCaseInsensitiveSearch] == NSOrderedSame)
        return sCallerId;
       
    return [NSString stringWithFormat:@"%@-%@", sCallerName, sCallerId];
}

- (void) ShowAlert :(NSString*) sCallerName :(NSString*) sCallerId;
{
    [self HideAlert];
    
    NSString* sTitle = [self PrepareTitle :sCallerName :sCallerId];
    
    if ( UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPad )
    {
        // if user's device is iPad.
        
        mIncomingCallAlert = [UIAlertController alertControllerWithTitle:sTitle message:@"" preferredStyle:UIAlertControllerStyleAlert];
    }
    else
    {
         mIncomingCallAlert = [UIAlertController alertControllerWithTitle:sTitle message:@"" preferredStyle:UIAlertControllerStyleActionSheet];
    }
    
   
    
    UIAlertAction* RejectCallAction = [UIAlertAction actionWithTitle:@"Reject Call" style:UIAlertActionStyleDefault handler:^(UIAlertAction * action)
                                    {
                                        [self->mCallAlertDelegate OnIncomingCallAlertReject];
                                        [self HideAlert];
                                    }];
    
    UIAlertAction* AcceptCallAction = [UIAlertAction actionWithTitle:@"Accept Call" style:UIAlertActionStyleDefault handler:^(UIAlertAction * action)
                                    {
                                        [self->mCallAlertDelegate OnIncomingCallAlertAccept];
                                        [cMainTabBarController SwitchToVideoTab :@""];
                                        
                                        [self HideAlert];
                                    }];
    
    UIAlertAction* MuteRingAction = [UIAlertAction actionWithTitle:@"Mute Ring" style:UIAlertActionStyleDefault handler:^(UIAlertAction * action)
                                    {
                                        [self->mCallAlertDelegate OnIncomingCallAlertMute];
                                        [self HideAlert];
                                     }];
    
    UIAlertAction* CancelAction = [UIAlertAction actionWithTitle:@"Cancel" style:UIAlertActionStyleDefault handler:^(UIAlertAction * action)
                                    {
                                        [self HideAlert];
                                    }];
    
    [RejectCallAction setValue:[UIColor redColor] forKey:@"titleTextColor"];
    
    [mIncomingCallAlert addAction:RejectCallAction];
    [mIncomingCallAlert addAction:AcceptCallAction];
    [mIncomingCallAlert addAction:MuteRingAction];
    [mIncomingCallAlert addAction:CancelAction];
    
     UIViewController* pMainTabBar = [cMainTabBarController GetMainTabBar];
    
    UIPopoverPresentationController* popoverPresentationController = mIncomingCallAlert.popoverPresentationController;
    
    if(popoverPresentationController != NULL)
    {
        [mIncomingCallAlert.popoverPresentationController setPermittedArrowDirections: 0];
        popoverPresentationController.sourceView = pMainTabBar.view;
        CGRect rect = pMainTabBar.view.frame;
        
        rect.origin.x = pMainTabBar.view.frame.size.width / 20;
        rect.origin.y = pMainTabBar.view.frame.size.height / 20;
        
        mIncomingCallAlert.popoverPresentationController.sourceRect = rect;
        mIncomingCallAlert.popoverPresentationController.sourceView = pMainTabBar.view;
    }
    
    [pMainTabBar presentViewController :mIncomingCallAlert animated:YES completion:nil];
    
    [self NotifyIncomingCall :sTitle];
}

- (void) HideAlert;
{
    if(mIncomingCallAlert == NULL)
        return;
    
    [mIncomingCallAlert dismissViewControllerAnimated :YES completion:nil];
    mIncomingCallAlert = NULL;
}

- (void) NotifyIncomingCall :(NSString*) sTitle;
{
    UILocalNotification* Notify = [[UILocalNotification alloc] init];
    
    Notify.alertBody = [NSString stringWithFormat: @"Incoming Call: %@", sTitle];
    
    Notify.alertAction = @"Answer";
    Notify.soundName   = @"oldphone-mono-30s.caf";
    
    [[UIApplication sharedApplication] presentLocalNotificationNow: Notify];
}


@end
