
//  Created by VaxSoft
//  Copyright © 2025 VaxSoft. All rights reserved.

#import "cMainTabBarController.h"
#import "cChatContactTableView.h"
#import "cRecentTableView.h"
#import "cVideoView.h"
#import "cVaxPhoneSIP.h"

#define TAB_ITEM_INDEX_RECENT       0
#define TAB_ITEM_INDEX_VIDEO        1
#define TAB_ITEM_INDEX_CHAT         2
#define TAB_ITEM_INDEX_SETTING      3

@interface cMainTabBarController ()
@end

static cMainTabBarController* mpMainTabBar = NULL;

@implementation cMainTabBarController

- (void) viewDidLoad
{
    mpMainTabBar = self;
    [cMainTabBarController SwitchToVideoTab :@""];
    
    [cChatContactTableView LoadMissedCount];
    [cRecentTableView LoadMissedCount];
            
    [[UITabBar appearance] setTintColor:[UIColor whiteColor]];
    [super viewDidLoad];
}

- (void) viewDidAppear: (BOOL) animated
{
    [super viewDidAppear :animated];
}

- (void) didReceiveMemoryWarning
{
    [super didReceiveMemoryWarning];
}

- (BOOL) prefersStatusBarHidden
{
    return NO;
}

+ (void) DismissMainTabBar;
{
    if(mpMainTabBar == NULL)
        return;
    
    [mpMainTabBar dismissViewControllerAnimated :false completion:nil];
}

+ (UIViewController*) GetMainTabBar;
{
    return mpMainTabBar;
}

+ (void) SwitchTab :(int) nTabIndex;
{
    if(mpMainTabBar == NULL)
        return;
    
    mpMainTabBar.selectedIndex = nTabIndex;
}

+ (void) SwitchToVideoTab :(NSString*) sPhoneNo;
{
    [cVideoView ShowDialpadWhenAppear :sPhoneNo];
    [cMainTabBarController SwitchTab :TAB_ITEM_INDEX_VIDEO];
}

+ (Boolean) IsChatContactSelected;
{
    if(mpMainTabBar == NULL)
        return false;
    
    if([mpMainTabBar selectedIndex] != TAB_ITEM_INDEX_CHAT)
        return false;
    
    return true;
}


+ (void) SetChatContactBadgeCount :(int) nValue;
{
    if(mpMainTabBar == NULL)
        return;
    
    UITabBarItem* tabBarItem = [[mpMainTabBar tabBar].items objectAtIndex :TAB_ITEM_INDEX_CHAT];
    
    if(nValue == 0)
    {
        tabBarItem.badgeValue = 0;
    }
    else
    {
        tabBarItem.badgeValue = [NSString stringWithFormat :@"%d", nValue];
    }
}

+ (Boolean) IsRecentSelected;
{
    if(mpMainTabBar == NULL)
        return false;
    
    if([mpMainTabBar selectedIndex] != TAB_ITEM_INDEX_RECENT)
        return false;
    
    return true;
}

+ (void) SetRecentBadgeCount :(int) nValue;
{
    if(mpMainTabBar == NULL)
        return;
    
    UITabBarItem* tabBarItem = [[mpMainTabBar tabBar].items objectAtIndex :TAB_ITEM_INDEX_RECENT];
    
    if(nValue == 0)
    {
        tabBarItem.badgeValue = 0;
    }
    else
    {
        tabBarItem.badgeValue = [NSString stringWithFormat :@"%d", nValue];
    }
}

@end
