
//  Created by VaxSoft
//  Copyright © 2025 VaxSoft. All rights reserved.

#import "cVaxPhoneSIP.h"
#import "cRecentTableView.h"

#import "cRecentRowData.h"
#import "cRecentRowView.h"

#import "cVideoView.h"
#import "cMainTabBarController.h"

#import "cRecentSectionData.h"
#import "cStoreRecents.h"

static cRecentTableView* mpRecentTableView = NULL;
static int m_nRecentMissedCount = 0;

@interface cRecentTableView ()

@end

@implementation cRecentTableView;

- (void) viewDidLoad
{
    m_bDidAppear = false;
    m_bEditingActivated = false;
    
    [super viewDidLoad];
}

- (void) didReceiveMemoryWarning
{
    [super didReceiveMemoryWarning];
}

- (void) LoadCallHistoryAll;
{
    cStoreRecents* pStoreDB = [cStoreRecents new];
    NSArray* pInfoAll = [pStoreDB GetCallHistoryAll];
    
    for(long nCount = 0; nCount < [pInfoAll count]; nCount++)
    {
        cStoreDataRecents* pData = [pInfoAll objectAtIndex :nCount];
        [cRecentTableView PostAddCallHistory :pData->m_nRecordId :pData->m_bOutboundCallType :pData->m_sName :pData->m_sPhoneNo :pData->m_nStartTime :pData->m_nEndTime :pData->m_nDuration :pData->m_nDayNo :pData->m_nHistoryTypeId];
    }
}

/////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////

- (void) viewDidAppear: (BOOL) animated
{
    m_bDidAppear = true;
    [super viewDidAppear :animated];
}

- (void) viewWillAppear: (BOOL) animated
{
    mpRecentTableView = self;
    m_bDidAppear = false;
    
    [self ActivateEditingMode :NO :NO];
    mpTableData = [NSMutableArray new];
    
    [self LoadCallHistoryAll];
    
    [cRecentTableView SetMissedCount :0];
    [self.tableView reloadData];
    
    [super viewWillAppear :animated];
}

- (void) viewWillDisappear: (BOOL)animated;
{
    mpRecentTableView = nil;
    
    [mpTableData removeAllObjects];
    mpTableData = nil;
    
    [super viewWillDisappear :animated];
}

//////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////

+ (void) PostAddCallHistory :(long) nRecordId :(bool) bOutboundCallType :(NSString*) sName :(NSString*) sPhoneNo :(long) nStartTime :(long) nEndTime :(long) nDuration :(long) nDayNo :(long) nHistoryTypeId;
{
    if(![cMainTabBarController IsRecentSelected] && bOutboundCallType == false && nHistoryTypeId == VAX_CALL_HISTORY_TYPE_MISSED)
    {
        [cRecentTableView SetMissedCount :-1];
    }
    
    if(mpRecentTableView == nil)
        return;
    
    [mpRecentTableView AddCallHistory :nRecordId :bOutboundCallType :sName :sPhoneNo :nStartTime :nEndTime :nDuration :nDayNo :nHistoryTypeId];
}

- (void) AddCallHistory :(long) nRecordId :(bool) bOutboundCallType :(NSString*) sName :(NSString*) sPhoneNo :(long) nStartTime :(long) nEndTime :(long) nDuration :(long) nDayNo :(long) nHistoryTypeId;
{
    long nSectionCountOld = [self GetSectionCount];
    
    long nSectionNo = [self AddSection :nDayNo];
    long nSectionCountNew = [self GetSectionCount];
    
    cRecentSectionData* pSection = [mpTableData objectAtIndex :nSectionNo];
    long nRowNo = [pSection AddRow :nRecordId :bOutboundCallType :sName :sPhoneNo :nStartTime :nEndTime :nDuration :nHistoryTypeId];
    
    if(m_bDidAppear)
    {
        [self.tableView beginUpdates];
        
        NSIndexPath* indexPath = [NSIndexPath indexPathForRow:nRowNo inSection:nSectionNo];
        [self.tableView insertRowsAtIndexPaths:@[indexPath] withRowAnimation:UITableViewRowAnimationAutomatic];
        
        if(nSectionCountOld != nSectionCountNew)
        {
            [self.tableView insertSections :[NSIndexSet indexSetWithIndex:nSectionNo] withRowAnimation:UITableViewRowAnimationFade];
        }
        
        [self.tableView endUpdates];
    }
}

//////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////

- (long) AddSection :(long) nDayNo;
{
    cRecentSectionData* pSection = nil;
    
    for(long nSectionNo = 0; nSectionNo < [mpTableData count]; nSectionNo++)
    {
        pSection = [mpTableData objectAtIndex :nSectionNo];
        long nLastDayNo = [pSection GetDayNo];
        
        if(nLastDayNo == nDayNo)
        {
            return nSectionNo;
        }
    }
    
    pSection = [[cRecentSectionData alloc] initWithDayNo :nDayNo];
    [mpTableData insertObject: pSection atIndex:0];
    
    int nSectionNo = 0;
    return nSectionNo;
}

//////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////

- (long) GetSectionCount;
{
    return [mpTableData count];
}

- (long) GetRowCount :(long) nSectionNo;
{
    if(nSectionNo >= [self GetSectionCount])
        return 0;
    
    cRecentSectionData* pSection = [mpTableData objectAtIndex :nSectionNo];
    return [pSection GetRowCount];
}

//////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////

- (NSString*) GetSectionTitle :(long) nSectionNo;
{
    if(nSectionNo >= [self GetSectionCount])
        return @"";
    
    cRecentSectionData* pSection = [mpTableData objectAtIndex :nSectionNo];
    return [pSection GetTitle];
}

- (cRecentRowData*) GetRowData :(long) nSectionNo :(long) nRowNo;
{
    if(nSectionNo >= [self GetSectionCount])
        return nil;
    
    cRecentSectionData* pSection = [mpTableData objectAtIndex :nSectionNo];
    return [pSection GetRowData :nRowNo];
}

//////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////

- (void) DeleteRow :(long) nSectionNo :(long) nRowNo;
{
    if(nSectionNo >= [self GetSectionCount])
        return;
    
    cRecentRowData* pRowData = [self GetRowData :nSectionNo :nRowNo];
    
    if(pRowData != nil)
    {
        long nRecordId = [pRowData GetRecordId];
        
        cStoreRecents* pStoreDB = [cStoreRecents new];
        [pStoreDB RemoveCallHistory :nRecordId];
    }
    
    cRecentSectionData* pSection = [mpTableData objectAtIndex :nSectionNo];
    [pSection DeleteRow :nRowNo];
    
    [self.tableView beginUpdates];
    
    NSIndexPath* indexPath = [NSIndexPath indexPathForRow:nRowNo inSection:nSectionNo];
    [self.tableView deleteRowsAtIndexPaths:@[indexPath] withRowAnimation:UITableViewRowAnimationFade];
    
    if([pSection GetRowCount] == 0)
    {
        [mpTableData removeObjectAtIndex :nSectionNo];
        [self.tableView deleteSections :[NSIndexSet indexSetWithIndex:nSectionNo] withRowAnimation:UITableViewRowAnimationFade];
    }
    
    [self.tableView endUpdates];
}

- (void) DeleteAll;
{
    cStoreRecents* pStoreDB = [cStoreRecents new];
    [pStoreDB RemoveCallHistoryAll];
    
    [mpTableData removeAllObjects];
}

//////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////

#pragma mark - Table view data source

- (NSInteger) numberOfSectionsInTableView:(UITableView *)tableView
{
    [self UpdateEditBarButton];
    return [self GetSectionCount];
}

- (NSInteger) tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section
{
    [self UpdateEditBarButton];
    return [self GetRowCount :section];
}

- (NSString *) tableView:(UITableView *)tableView titleForHeaderInSection:(NSInteger)section
{
    return [self GetSectionTitle :section];
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
    cRecentRowData* pRowData = [self GetRowData :indexPath.section :indexPath.row];
    cRecentRowView* pRowView = [tableView dequeueReusableCellWithIdentifier:@"RecentCellId" forIndexPath:indexPath];
        
    [pRowView SetRowData: pRowData];
    return pRowView;
}

- (void) tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath
{
    cRecentRowData* pRowData = [self GetRowData :indexPath.section :indexPath.row];
    
    [UIView beginAnimations:nil context:NULL];
    [UIView setAnimationDuration: 0.65];
    [UIView setAnimationTransition:UIViewAnimationTransitionFlipFromLeft forView: self.view.window cache:YES];
    
    [cMainTabBarController SwitchToVideoTab :[pRowData GetPhoneNo]];
    [UIView commitAnimations];
}

//////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////

- (IBAction) OnClickClear:(id)sender
{
    [self DeleteAll];
    [self.tableView reloadData];
}

- (IBAction) OnClickEdit:(id)sender
{
    [self ActivateEditingMode :!m_bEditingActivated :NO];
    [self UpdateEditBarButton];
}

- (BOOL)tableView:(UITableView *)tableView canEditRowAtIndexPath:(NSIndexPath *)indexPath
{
    return YES;
}

- (void) UpdateEditBarButton;
{
    if([self GetSectionCount] == 0)
    {
        self.navigationItem.leftBarButtonItem = nil;
        self.navigationItem.rightBarButtonItem = nil;
        
        [self ActivateEditingMode :NO :NO];
    }
    else if(m_bEditingActivated)
    {
        self.navigationItem.leftBarButtonItem = nil;
        self.navigationItem.leftBarButtonItem = mDoneBarBtn;
        self.navigationItem.rightBarButtonItem = mClearBarBtn;
    }
    else
    {
        self.navigationItem.leftBarButtonItem = nil;
        self.navigationItem.leftBarButtonItem = mEditBarBtn;
        self.navigationItem.rightBarButtonItem = mClearBarBtn;
    }
}

- (void) tableView:(UITableView *)tableView commitEditingStyle:(UITableViewCellEditingStyle)editingStyle forRowAtIndexPath:(NSIndexPath *)indexPath
{
    if (editingStyle == UITableViewCellEditingStyleDelete)
    {
        [self DeleteRow :indexPath.section :indexPath.row];
    }
    else if (editingStyle == UITableViewCellEditingStyleInsert)
    {
        
    }   
}

- (void) ActivateEditingMode :(Boolean) bEnable :(Boolean) bAnimated;
{
    m_bEditingActivated = bEnable;
    [self setEditing:bEnable animated:bAnimated];
}

///////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////

+ (void) LoadMissedCount;
{
    m_nRecentMissedCount = [cRecentTableView ReadMissedCount];
    [cMainTabBarController SetRecentBadgeCount :m_nRecentMissedCount];
}

+ (void) SetMissedCount :(int) nValue;
{
    if(nValue == 0)
    {
        m_nRecentMissedCount = 0;
    }
    else
    {
        m_nRecentMissedCount++;
    }
    
    [cMainTabBarController SetRecentBadgeCount :m_nRecentMissedCount];
    [cRecentTableView StoreMissedCount];
}

+ (void) StoreMissedCount;
{
    NSUserDefaults* pRefs = [NSUserDefaults standardUserDefaults];
    
    [pRefs setObject :[NSNumber numberWithInt: m_nRecentMissedCount] forKey:@"m_nRecentMissedCount"];
    [pRefs synchronize];
}

+ (int) ReadMissedCount;
{
    NSUserDefaults* pRefs = [NSUserDefaults standardUserDefaults];
    
    NSNumber* pValue = [pRefs objectForKey :@"m_nRecentMissedCount"];
    if(pValue == nil) return 0;
    
    return [pValue intValue];
}

@end
