
//  Created by VaxSoft
//  Copyright © 2025 VaxSoft. All rights reserved.

#import "cVaxPhoneSIP.h"

#import "cSettingMicSpkTableView.h"
#import "cSettingMicSpkRowView.h"

#import "cStoreSpKMic.h"

#define SECTION_TOTAL_COUNT          2

#define SECTION_SPK_ROW_COUNT        1
#define SECTION_MIC_ROW_COUNT        2

#define SECTION_ID_SPK               0
#define SECTION_ID_MIC               1

#define ROW_ID_AUTO_GAIN_SPK         0

#define ROW_ID_AUTO_GAIN_MIC         0
#define ROW_ID_VOLUME_BOOST_MIC      1

@interface cSettingMicSpkTableView ()
@end

@implementation cSettingMicSpkTableView

- (void) viewDidLoad
{
    [super viewDidLoad];
}

- (void) didReceiveMemoryWarning
{
    [super didReceiveMemoryWarning];
}

//////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////

+ (void) ResetData;
{
    cStoreSpKMic* pStore = [cStoreSpKMic new];
    [pStore ResetData];
    
    [cSettingMicSpkTableView PostInitialize];
}

//////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////

+ (void) PostInitialize;
{
    cStoreSpKMic* pStore = [cStoreSpKMic new];
    
    ///////////////////////////////////////////////////////////////////////////////////
    
    Boolean bEnable = [pStore GetAutoGainSpk];
    [[cVaxPhoneMicSpk GetSelf] AutoGainSpk :bEnable];
    
    bEnable = [pStore GetAutoGainMic];
    [[cVaxPhoneMicSpk GetSelf] AutoGainMic :bEnable];
    
    ///////////////////////////////////////////////////////////////////////////////////
            
    bEnable = [pStore GetVolumeBoostMic];
    [[cVaxPhoneMicSpk GetSelf] SetVolumeBoostMic :bEnable];
}

/////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////

- (void) viewWillAppear: (BOOL) animated
{
    cStoreSpKMic* pStore = [cStoreSpKMic new];
    
    mbAutoGainSpk = [pStore GetAutoGainSpk];
    mbAutoGainMic = [pStore GetAutoGainMic];
    
    mbVolumeBoostMic = [pStore GetVolumeBoostMic];
           
    [self.tableView reloadData];
    [super viewWillAppear :animated];
}

//////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////

#pragma mark - Table view data source

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView
{
    return SECTION_TOTAL_COUNT;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section
{
    if(section == SECTION_ID_SPK)
        return SECTION_SPK_ROW_COUNT;
           
    return SECTION_MIC_ROW_COUNT;
}

- (NSString *)tableView:(UITableView *)tableView titleForHeaderInSection:(NSInteger)section
{
    if(section == SECTION_ID_SPK)
        return @"Speaker";
    
    return @"Microphone";
}

//////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
    if( indexPath.row == ROW_ID_AUTO_GAIN_SPK && indexPath.section == SECTION_ID_SPK)
    {
        cSettingMicSpkRowView* pRowView = [tableView dequeueReusableCellWithIdentifier:@"SpeakerAutoGainCellId" forIndexPath:indexPath];
        
        [pRowView UpdateView : mbAutoGainSpk];
        return pRowView;
    }
        
    if(indexPath.row == ROW_ID_AUTO_GAIN_MIC && indexPath.section == SECTION_ID_MIC)
    {
        cSettingMicSpkRowView* pRowView = [tableView dequeueReusableCellWithIdentifier:@"MicrophoneAutoGainCellId" forIndexPath:indexPath];
        
        [pRowView UpdateView : mbAutoGainMic];
        return pRowView;
    }
            
    cSettingMicSpkRowView* pRowView = [tableView dequeueReusableCellWithIdentifier:@"MicrophoneVolumeBoostCellId" forIndexPath:indexPath];
    
    [pRowView UpdateView : mbVolumeBoostMic];
    return pRowView;
}

//////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////

- (IBAction) OnClickAutoGainSpk :(id)sender
{
    cCustomCellCheckBox* pCheckBox = (cCustomCellCheckBox*) sender;
    Boolean bAutoGainSpk = [pCheckBox GetState];
    
    [[cVaxPhoneMicSpk GetSelf] AutoGainSpk :bAutoGainSpk];
    
    cStoreSpKMic* pStore = [cStoreSpKMic new];
    [pStore SetAutoGainSpk :bAutoGainSpk];
}

-(IBAction) OnClickAutoGainMic :(id)sender
{
    cCustomCellCheckBox* pCheckBox = (cCustomCellCheckBox*) sender;
    Boolean bAutoGainMic = [pCheckBox GetState];
    
    [[cVaxPhoneMicSpk GetSelf] AutoGainMic :bAutoGainMic];
    
    cStoreSpKMic* pStore = [cStoreSpKMic new];
    [pStore SetAutoGainMic :bAutoGainMic];
}

-(IBAction) OnClickVolBoostMic :(id)sender
{
    cCustomCellCheckBox* pCheckBox = (cCustomCellCheckBox*) sender;
    Boolean bVolumeBoostMic = [pCheckBox GetState];
    
    [[cVaxPhoneMicSpk GetSelf] SetVolumeBoostMic :bVolumeBoostMic];
    
    cStoreSpKMic* pStore = [cStoreSpKMic new];
    [pStore SetVolumeBoostMic :bVolumeBoostMic];
}

//////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////

@end
