
//  Created by VaxSoft
//  Copyright © 2025 VaxSoft. All rights reserved.

#import "cVaxPhoneSIP.h"

#import "cSettingVideoCodecTableView.h"
#import "cSettingVideoCodecRowView.h"

#import "cStoreVideoCodecs.h"

#define SECTION_TOTAL_COUNT       1
#define ROW_TOTAL_COUNT           VAX_VIDEO_CODEC_TOTAL

#define SECTION_ID_VIDEO_CODEC    0

//////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////

@interface cSettingVideoCodecTableView ()
@end

@implementation cSettingVideoCodecTableView

- (void)viewDidLoad
{
    [super viewDidLoad];
    [self.tableView setEditing:YES animated:YES];
}

- (void)didReceiveMemoryWarning
{
    [super didReceiveMemoryWarning];
}

//////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////

+ (void) PostInitialize;
{
    [cSettingVideoCodecTableView ApplyVideoCodec];
}

+ (void) ResetData;
{
    cStoreVideoCodecs* pStore = [cStoreVideoCodecs new];
    [pStore ResetData];
    
    [cSettingVideoCodecTableView PostInitialize];
}

+ (void) ApplyVideoCodec;
{
    NSMutableArray* aCodecNo = [NSMutableArray new];
    
    cStoreVideoCodecs* pStore = [cStoreVideoCodecs new];
    [pStore GetVideoCodec :aCodecNo];
    
    [[cVaxPhoneCodec GetSelf] ActivateVideoCodec :false : -1];
    
    for(int nCount = 0; nCount < VAX_VIDEO_CODEC_TOTAL; nCount++)
    {
        int nCodecNo = [[aCodecNo objectAtIndex :nCount] intValue];
        Boolean bEnabled = [pStore GetVideoCodecEnabled :nCodecNo];
        
        [[cVaxPhoneCodec GetSelf] ActivateVideoCodec :bEnabled :nCodecNo];
    }
}

/////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////

- (void) viewWillAppear: (BOOL) animated
{
    mCodecNo = [NSMutableArray new];
    
    cStoreVideoCodecs* pStore = [cStoreVideoCodecs new];
    [pStore GetVideoCodec :mCodecNo];
    
    [[cVaxPhoneCodec GetSelf] ActivateVideoCodec :false : -1];
    
    for(int nCount = 0; nCount < VAX_VIDEO_CODEC_TOTAL; nCount++)
    {
        int nCodecNo = [[mCodecNo objectAtIndex :nCount] intValue];
        mCodecEnabled[nCodecNo] = [pStore GetVideoCodecEnabled :nCodecNo];
        
        [[cVaxPhoneCodec GetSelf] ActivateVideoCodec :mCodecEnabled[nCodecNo] :nCodecNo];
    }
    
    [self.tableView reloadData];
    [super viewWillAppear :animated];
}

//////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////

#pragma mark - Table view data source

- (NSInteger) numberOfSectionsInTableView:(UITableView *)tableView
{
    return SECTION_TOTAL_COUNT;
}

- (NSInteger) tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section
{
    return ROW_TOTAL_COUNT;
}

//////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////

- (NSString*) GenCellName: (long) nCodecNo;
{
    return @"VideoCodecVP8CellId";
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
    long nCodecNo = [[mCodecNo objectAtIndex :indexPath.row] longValue];
    NSString* sCellName = [self GenCellName :nCodecNo];
    
    cSettingVideoCodecRowView* pRowView = [tableView dequeueReusableCellWithIdentifier:sCellName forIndexPath:indexPath];
    
    Boolean bCodecEnabled = mCodecEnabled[nCodecNo];
    [pRowView SetCodecEnabled :bCodecEnabled];
    
    return pRowView;
}

//////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////

- (BOOL) tableView:(UITableView *)tableView canMoveRowAtIndexPath:(NSIndexPath *)indexPath
{
    return NO;
}

- (UITableViewCellEditingStyle) tableView :(UITableView *)tableView editingStyleForRowAtIndexPath:(NSIndexPath *)indexPath
{
    return UITableViewCellEditingStyleNone;
}

- (BOOL) tableView:(UITableView *)tableView shouldIndentWhileEditingRowAtIndexPath:(NSIndexPath *)indexPath
{
    return NO;
}

//////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////

- (void)tableView:(UITableView *)tableView moveRowAtIndexPath:(NSIndexPath *)sourceIndexPath toIndexPath:(NSIndexPath *)destinationIndexPath
{
    long nSrcRowNo = sourceIndexPath.row;
    long nDscRowNo = destinationIndexPath.row;
    
    NSNumber* pCodecNo = [mCodecNo objectAtIndex :nSrcRowNo];
    [mCodecNo removeObjectAtIndex :nSrcRowNo];
    [mCodecNo insertObject :pCodecNo atIndex:nDscRowNo];
    
    cStoreVideoCodecs* pStore = [cStoreVideoCodecs new];
    [pStore SetVideoCodec :mCodecNo];
    
    [cSettingVideoCodecTableView ApplyVideoCodec];
}

//////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////

- (IBAction) OnClickVideoCodecVP8 :(id)sender
{
    cCustomCellCheckBox* pCheckBox = (cCustomCellCheckBox*) sender;
    Boolean bEnabled = [pCheckBox GetState];
    
    mCodecEnabled[VAX_VIDEO_CODEC_VP8] = bEnabled;
    
    cStoreVideoCodecs* pStore = [cStoreVideoCodecs new];
    [pStore SetVideoCodecEnabled :VAX_VIDEO_CODEC_VP8 :bEnabled];
    
    [[cVaxPhoneCodec GetSelf] ActivateVideoCodec :bEnabled :VAX_VIDEO_CODEC_VP8];
}

@end
