
//  Created by VaxSoft
//  Copyright © 2025 VaxSoft. All rights reserved.

#import "cVaxPhoneSIP.h"

#import "cSettingVoiceCodecTableView.h"
#import "cSettingVoiceCodecRowView.h"

#import "cStoreVoiceCodecs.h"
#import "cCustomCellCheckBox.h"

#define SECTION_TOTAL_COUNT        1
#define ROW_TOTAL_COUNT            VAX_AUDIO_CODEC_TOTAL

/////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////

@interface cSettingVoiceCodecTableView ()
@end

@implementation cSettingVoiceCodecTableView

- (void)viewDidLoad
{
    [super viewDidLoad];
    [self.tableView setEditing:YES animated:YES];
}

- (void)didReceiveMemoryWarning
{
    [super didReceiveMemoryWarning];
}

//////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////

+ (void) PostInitialize;
{
    [cSettingVoiceCodecTableView ApplyAudioCodec];
}

+ (void) ResetData;
{
    cStoreVoiceCodecs* pStore = [cStoreVoiceCodecs new];
    [pStore ResetData];
    
    [cSettingVoiceCodecTableView PostInitialize];
}

+ (void) ApplyAudioCodec;
{
    NSMutableArray* aCodecNo = [NSMutableArray new];
    
    cStoreVoiceCodecs* pStore = [cStoreVoiceCodecs new];
    [pStore GetVoiceCodec :aCodecNo];
    
    [[cVaxPhoneCodec GetSelf] ActivateAudioCodec :false : -1];
    
    for(int nCount = 0; nCount < VAX_AUDIO_CODEC_TOTAL; nCount++)
    {
        int nCodecNo = [[aCodecNo objectAtIndex :nCount] intValue];
        Boolean bEnabled = [pStore GetVoiceCodecEnabled :nCodecNo];
        
        [[cVaxPhoneCodec GetSelf] ActivateAudioCodec :bEnabled :nCodecNo];
    }
}

/////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////

- (void) viewWillAppear: (BOOL) animated
{
    mCodecNo = [NSMutableArray new];
    
    cStoreVoiceCodecs* pStore = [cStoreVoiceCodecs new];
    [pStore GetVoiceCodec :mCodecNo];
    
    [[cVaxPhoneCodec GetSelf] ActivateAudioCodec :false : -1];
    
    for(int nCount = 0; nCount < VAX_AUDIO_CODEC_TOTAL; nCount++)
    {
        int nCodecNo = [[mCodecNo objectAtIndex :nCount] intValue];
        mCodecEnabled[nCodecNo] = [pStore GetVoiceCodecEnabled :nCodecNo];
        
        [[cVaxPhoneCodec GetSelf] ActivateAudioCodec :mCodecEnabled[nCodecNo] :nCodecNo];
    }
    
    [self.tableView reloadData];
    [super viewWillAppear :animated];
}

//////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////

#pragma mark - Table view data source

- (NSInteger) numberOfSectionsInTableView:(UITableView *)tableView
{
    return SECTION_TOTAL_COUNT;
}

- (NSInteger) tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section
{
    return ROW_TOTAL_COUNT;
}

//////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////

- (NSString*) GenCellName: (long) nCodecNo;
{
    if(nCodecNo == VAX_AUDIO_CODEC_G711U)
        return @"AudioCodecG711uCellId";
    
    if(nCodecNo == VAX_AUDIO_CODEC_G711A)
        return @"AudioCodecG711aCellId";
    
    if(nCodecNo == VAX_AUDIO_CODEC_G729)
        return @"AudioCodecG729CellId";
    
    if(nCodecNo == VAX_AUDIO_CODEC_GSM610)
        return @"AudioCodecGSMCellId";
    
    return @"AudioCodeciLBCCellId";
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
    long nCodecNo = [[mCodecNo objectAtIndex :indexPath.row] longValue];
    NSString* sCellName = [self GenCellName :nCodecNo];
    
    cSettingVoiceCodecRowView* pRowView = [tableView dequeueReusableCellWithIdentifier:sCellName forIndexPath:indexPath];
    
    Boolean bCodecEnabled = mCodecEnabled[nCodecNo];
    [pRowView SetCodecEnabled :bCodecEnabled];
    
    return pRowView;
}

//////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////

- (BOOL) tableView:(UITableView *)tableView canMoveRowAtIndexPath:(NSIndexPath *)indexPath
{
    return YES;
}

- (UITableViewCellEditingStyle) tableView :(UITableView *)tableView editingStyleForRowAtIndexPath:(NSIndexPath *)indexPath
{
    return UITableViewCellEditingStyleNone;
}

- (BOOL) tableView:(UITableView *)tableView shouldIndentWhileEditingRowAtIndexPath:(NSIndexPath *)indexPath
{
    return NO;
}

//////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////

- (void)tableView:(UITableView *)tableView moveRowAtIndexPath:(NSIndexPath *)sourceIndexPath toIndexPath:(NSIndexPath *)destinationIndexPath
{
    long nSrcRowNo = sourceIndexPath.row;
    long nDscRowNo = destinationIndexPath.row;
    
    NSNumber* pCodecNo = [mCodecNo objectAtIndex :nSrcRowNo];
    [mCodecNo removeObjectAtIndex :nSrcRowNo];
    [mCodecNo insertObject :pCodecNo atIndex:nDscRowNo];
    
    cStoreVoiceCodecs* pStore = [cStoreVoiceCodecs new];
    [pStore SetVoiceCodec :mCodecNo];
    
    [cSettingVoiceCodecTableView ApplyAudioCodec];
}

//////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////

- (IBAction) OnClickVideoCodecG711u :(id)sender
{
    cCustomCellCheckBox* pCheckBox = (cCustomCellCheckBox*) sender;
    Boolean bEnabled = [pCheckBox GetState];
    
    mCodecEnabled[VAX_AUDIO_CODEC_G711U] = bEnabled;
    
    cStoreVoiceCodecs* pStore = [cStoreVoiceCodecs new];
    [pStore SetVoiceCodecEnabled :VAX_AUDIO_CODEC_G711U :bEnabled];
    
    [[cVaxPhoneCodec GetSelf] ActivateAudioCodec :bEnabled :VAX_AUDIO_CODEC_G711U];
}

- (IBAction) OnClickVideoCodecG711a :(id)sender
{
    cCustomCellCheckBox* pCheckBox = (cCustomCellCheckBox*) sender;
    Boolean bEnabled = [pCheckBox GetState];
    
    mCodecEnabled[VAX_AUDIO_CODEC_G711A] = bEnabled;
    
    cStoreVoiceCodecs* pStore = [cStoreVoiceCodecs new];
    [pStore SetVoiceCodecEnabled :VAX_AUDIO_CODEC_G711A :bEnabled];
    
    [[cVaxPhoneCodec GetSelf] ActivateAudioCodec :bEnabled :VAX_AUDIO_CODEC_G711A];
}

- (IBAction) OnClickVideoCodecG729 :(id)sender
{
    cCustomCellCheckBox* pCheckBox = (cCustomCellCheckBox*) sender;
    Boolean bEnabled = [pCheckBox GetState];
    
    mCodecEnabled[VAX_AUDIO_CODEC_G729] = bEnabled;
    
    cStoreVoiceCodecs* pStore = [cStoreVoiceCodecs new];
    [pStore SetVoiceCodecEnabled :VAX_AUDIO_CODEC_G729 :bEnabled];
    
    [[cVaxPhoneCodec GetSelf] ActivateAudioCodec :bEnabled :VAX_AUDIO_CODEC_G729];
}

- (IBAction) OnClickVideoCodecGSM :(id)sender
{
    cCustomCellCheckBox* pCheckBox = (cCustomCellCheckBox*) sender;
    Boolean bEnabled = [pCheckBox GetState];
    
    mCodecEnabled[VAX_AUDIO_CODEC_GSM610] = bEnabled;
    
    cStoreVoiceCodecs* pStore = [cStoreVoiceCodecs new];
    [pStore SetVoiceCodecEnabled :VAX_AUDIO_CODEC_GSM610 :bEnabled];
    
    [[cVaxPhoneCodec GetSelf] ActivateAudioCodec :bEnabled :VAX_AUDIO_CODEC_GSM610];
}

- (IBAction) OnClickVideoCodeciLBC :(id)sender
{
    cCustomCellCheckBox* pCheckBox = (cCustomCellCheckBox*) sender;
    Boolean bEnabled = [pCheckBox GetState];
    
    mCodecEnabled[VAX_AUDIO_CODEC_ILBC] = bEnabled;
    
    cStoreVoiceCodecs* pStore = [cStoreVoiceCodecs new];
    [pStore SetVoiceCodecEnabled :VAX_AUDIO_CODEC_ILBC :bEnabled];
    
    [[cVaxPhoneCodec GetSelf] ActivateAudioCodec :bEnabled :VAX_AUDIO_CODEC_ILBC];
}

@end

