
//  Created by VaxSoft
//  Copyright © 2025 VaxSoft. All rights reserved.

#import "AppDelegate.h"
#import "cVaxPhoneSIP.h"

#import "cSettingRowView.h"
#import "cSettingsTableView.h"

#import "cCustomCellAccessory.h"
#import "cMainTabBarController.h"

#import "cStoreSettings.h"

#define SECTION_TOTAL_COUNT         4

#define SECTION_ID_VIDEO            0
#define SECTION_ID_VOICE            1
#define SECTION_ID_NETWORK          2
#define SECTION_ID_RESET_ALL        4

#define SECTION_VIDEO_ROWS          4
#define SECTION_VOICE_ROWS          3
#define SECTION_NETWORK_ROWS        3
#define SECTION_RESET_ALL_ROWS      1

#define ROW_ID_DIAGNOSTIC_LOG       1

@interface cSettingsTableView ()
@end

@implementation cSettingsTableView

- (void) viewDidLoad
{
    [super viewDidLoad];
}

- (void) didReceiveMemoryWarning
{
    [super didReceiveMemoryWarning];
}

//////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////

+ (void) PostInitialize;
{
    cStoreSettings* pStore = [cStoreSettings new];
        
    Boolean bDiagnosticLog = [pStore GetDiagnosticLog];
    [[cVaxPhoneLog GetSelf] DiagnosticLog :bDiagnosticLog];
}

/////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////

- (void) viewWillAppear: (BOOL) animated
{
    cStoreSettings* pStore = [cStoreSettings new];
    
    mbDiagnosticLog = [pStore GetDiagnosticLog];
    
    [self.tableView reloadData];
    [super viewWillAppear :animated];
}

//////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////

#pragma mark - Table view data source

- (NSInteger) numberOfSectionsInTableView :(UITableView *) tableView
{
    return SECTION_TOTAL_COUNT;
}

- (NSInteger) tableView :(UITableView *)tableView numberOfRowsInSection :(NSInteger) section
{
    if(section == SECTION_ID_VIDEO)
        return SECTION_VIDEO_ROWS;
    
    if(section == SECTION_ID_VOICE)
        return SECTION_VOICE_ROWS;
    
    if(section == SECTION_ID_NETWORK)
        return SECTION_NETWORK_ROWS;
                
    return SECTION_RESET_ALL_ROWS;
}

- (UITableViewCell *) tableView :(UITableView *)tableView cellForRowAtIndexPath :(NSIndexPath *) indexPath
{
    if(indexPath.row == 0 && indexPath.section == SECTION_ID_VIDEO)
    {
        UITableViewCell* pCell = [tableView dequeueReusableCellWithIdentifier:@"VideoBitrateCellId" forIndexPath:indexPath];
        pCell.accessoryView = [cCustomCellAccessory new];
        
        return pCell;
    }
    
    if(indexPath.row == 1 && indexPath.section == SECTION_ID_VIDEO)
    {
        UITableViewCell* pCell = [tableView dequeueReusableCellWithIdentifier:@"VideoQualityCellId" forIndexPath:indexPath];
        pCell.accessoryView = [cCustomCellAccessory new];
        
        return pCell;
    }
    
    if(indexPath.row == 2 && indexPath.section == SECTION_ID_VIDEO)
    {
        UITableViewCell* pCell = [tableView dequeueReusableCellWithIdentifier:@"VideoCodecCellId" forIndexPath:indexPath];
        pCell.accessoryView = [cCustomCellAccessory new];
        
        return pCell;
    }
    
    if(indexPath.row == 3 && indexPath.section == SECTION_ID_VIDEO)
    {
        UITableViewCell* pCell = [tableView dequeueReusableCellWithIdentifier:@"VoiceCodecCellId" forIndexPath:indexPath];
        pCell.accessoryView = [cCustomCellAccessory new];
        
        return pCell;
    }
    
    if(indexPath.row == 0 && indexPath.section == SECTION_ID_VOICE)
    {
        UITableViewCell* pCell = [tableView dequeueReusableCellWithIdentifier:@"MicSpkCellId" forIndexPath:indexPath];
        pCell.accessoryView = [cCustomCellAccessory new];
        
        return pCell;
    }
    
    if(indexPath.row == 1 && indexPath.section == SECTION_ID_VOICE)
    {
        UITableViewCell* pCell = [tableView dequeueReusableCellWithIdentifier:@"RingtoneCellId" forIndexPath:indexPath];
        pCell.accessoryView = [cCustomCellAccessory new];
        
        return pCell;
    }
    
    if(indexPath.row == 2 && indexPath.section == SECTION_ID_VOICE)
    {
        UITableViewCell* pCell = [tableView dequeueReusableCellWithIdentifier:@"VoiceChangerCellId" forIndexPath:indexPath];
        pCell.accessoryView = [cCustomCellAccessory new];
        
        return pCell;
    }
    
    if(indexPath.row == 0 && indexPath.section == SECTION_ID_NETWORK)
    {
        UITableViewCell* pCell = [tableView dequeueReusableCellWithIdentifier:@"NetworkCellId" forIndexPath:indexPath];
        pCell.accessoryView = [cCustomCellAccessory new];
        
        return pCell;
    }
    
    if(indexPath.row == 1 && indexPath.section == SECTION_ID_NETWORK)
    {
        UITableViewCell* pCell = [tableView dequeueReusableCellWithIdentifier:@"DigitDTMFSettingsCellId" forIndexPath:indexPath];
        pCell.accessoryView = [cCustomCellAccessory new];
        
        return pCell;
    }
        
    if(indexPath.row == 2 && indexPath.section == SECTION_ID_NETWORK)
    {
        cSettingRowView* pRowView = [tableView dequeueReusableCellWithIdentifier:@"DiagnosticCellId" forIndexPath:indexPath];
        [pRowView UpdateDiagnosticLogView :mbDiagnosticLog];
        
        return pRowView;
    }
    
    return [tableView dequeueReusableCellWithIdentifier:@"ResetAllSettingsCellId" forIndexPath:indexPath];
}

//////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////

- (void) ShowResetAlert;
{
    UIAlertController* MsgAlert = [UIAlertController alertControllerWithTitle:@"Reset Settings" message:@"" preferredStyle:UIAlertControllerStyleAlert];
    
    UIAlertAction* ResetAction = [UIAlertAction actionWithTitle:@"Reset All" style:UIAlertActionStyleDefault handler:^(UIAlertAction * action)
                                       {
                                           [AppDelegate ResetSettings];
                                           [self.tableView reloadData];
                                       }];
    
    UIAlertAction* CancelAction = [UIAlertAction actionWithTitle:@"Cancel" style:UIAlertActionStyleDefault handler:^(UIAlertAction * action)
                                       {
                                        
                                       }];
    
    [ResetAction setValue:[UIColor redColor] forKey:@"titleTextColor"];
    
    [MsgAlert addAction:ResetAction];
    [MsgAlert addAction:CancelAction];
    
    [self presentViewController :MsgAlert animated:YES completion:nil];
}

- (IBAction) OnClickResetSettings :(id)sender
{
    [self ShowResetAlert];
    
    
}

//////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////

- (IBAction) OnClickDiagnosticLog :(id)sender
{
    cCustomCellCheckBox* pCheckBox = (cCustomCellCheckBox*) sender;
    Boolean bDiagnosticLog = [pCheckBox GetState];
    
    [[cVaxPhoneLog GetSelf] DiagnosticLog :bDiagnosticLog];
    
    cStoreSettings* pStore = [cStoreSettings new];
    [pStore SetDiagnosticLog :bDiagnosticLog];
}

//////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////

- (IBAction) OnClickLogout :(id)sender
{
    [[cVaxPhoneSIP GetSelf] DisconnectCall];
    [cMainTabBarController DismissMainTabBar];
}

@end
