
//  Created by VaxSoft
//  Copyright © 2025 VaxSoft. All rights reserved.

#import "cVideoRemoteView.h"
#import "cVaxPhoneSIP.h"

static cVideoRemoteView* mpVideoRemoteView = NULL;
static Boolean mbVideoRemoteHideDisplayWhenAppear = true;
static int mbVideoRemoteDisplayGravity = 0;
static Boolean mbVideoRemoteCallConnected = false;

@interface cVideoRemoteView ()

@end

@implementation cVideoRemoteView

- (void)viewDidLoad
{
    mDisplayLayer = [CALayer layer];
    
    m_nLastWidth  = -1;
    m_nLastHeight = -1;
    
    mBgColor = self.view.backgroundColor;
    
    [super viewDidLoad];
}

- (void) viewWillAppear:(BOOL)animated
{
    mpVideoRemoteView = self;
    [mDisplayLayer setHidden :mbVideoRemoteHideDisplayWhenAppear];
    
    [self ApplyContentGravity];
    [super viewWillAppear :animated];
}

- (void) viewDidAppear:(BOOL)animated
{
    [super viewDidAppear :animated];
    [self.view.layer addSublayer :mDisplayLayer];
}

- (void) viewWillDisappear:(BOOL)animated
{
    mpVideoRemoteView = NULL;
    [super viewWillDisappear :animated];
}

- (void)didReceiveMemoryWarning
{
    [super didReceiveMemoryWarning];
}

//////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////

- (void) OnAdjustSize;
{
    mDisplayLayer.bounds = CGRectMake(0, 0, self.view.frame.size.width, self.view.frame.size.height);
    mDisplayLayer.position = CGPointMake(self.view.frame.size.width/2.0, self.view.frame.size.height/2.0);
    
    [self ApplyContentGravity];
}

- (void) AdjustSize :(Boolean) bAfterDelay;
{
    [NSObject cancelPreviousPerformRequestsWithTarget:self selector:@selector(OnAdjustSize) object:nil];
    
    if(bAfterDelay)
        [self performSelector :@selector(OnAdjustSize) withObject:nil afterDelay:0.5];
    else
        [self OnAdjustSize];
}

//////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////

- (void) viewWillTransitionToSize:(CGSize)size withTransitionCoordinator:(id<UIViewControllerTransitionCoordinator>)coordinator ;
{
    [self AdjustSize :true];
}

- (void) DisplayFrame :(char*)pDataRGB :(int)nWidth :(int)nHeight;
{
    if(m_nLastWidth != nWidth || m_nLastHeight != nHeight)
    {
        [self AdjustSize :true];
        
        m_nLastWidth = nWidth;
        m_nLastHeight = nHeight;
    }
    
    CGColorSpaceRef colorSpace = CGColorSpaceCreateDeviceRGB();
    CGContextRef context = CGBitmapContextCreate(pDataRGB, nWidth, nHeight, 8, nWidth * 4, colorSpace, kCGBitmapByteOrder32Little | kCGImageAlphaPremultipliedFirst);
    
    CGImageRef quartzImage = CGBitmapContextCreateImage (context);
    
    mDisplayLayer.contents = (__bridge id) quartzImage;
    
    CGImageRelease (quartzImage);
    CGContextRelease (context);
    CGColorSpaceRelease (colorSpace);
}

//////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////

- (void) OnConnectedCall;
{
    self.view.backgroundColor = mBgColor;
}

+ (void) PostConnectedCall;
{
    mbVideoRemoteCallConnected = true;
    mbVideoRemoteHideDisplayWhenAppear = false;
    
    if(mpVideoRemoteView == NULL)
        return;
    
    [mpVideoRemoteView OnConnectedCall];
}

//////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////

- (void) OnDisconnectedCall;
{
    self.view.backgroundColor = mBgColor;
    [mDisplayLayer setHidden :true];
}

+ (void) PostDisconnectedCall;
{
    mbVideoRemoteCallConnected = false;
    mbVideoRemoteHideDisplayWhenAppear = true;
    
    if(mpVideoRemoteView == NULL)
        return;
    
    [mpVideoRemoteView OnDisconnectedCall];
}

//////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////

- (void) OnPostedFrameRGB :(char*) pFrameRGB :(int) nFrameSize :(int) nFrameWidth :(int) nFrameHeight;
{
    if(mbVideoRemoteCallConnected)
    {
        if([mDisplayLayer isHidden])
            [mDisplayLayer setHidden :false];
    }
    
    [mpVideoRemoteView DisplayFrame :pFrameRGB :nFrameWidth :nFrameHeight];
}

+ (void) PostFrameRGB :(char*) pFrameRGB :(int) nFrameSize :(int) nFrameWidth :(int) nFrameHeight;
{
    if(mpVideoRemoteView == NULL)
        return;
    
    [mpVideoRemoteView OnPostedFrameRGB :pFrameRGB :nFrameSize :nFrameWidth :nFrameHeight];
}

//////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////

#define TOTAL_DISPLAY_GRAVITY_COUNT 4

- (void) ApplyContentGravity;
{
    if(mbVideoRemoteDisplayGravity < 0 || mbVideoRemoteDisplayGravity >= TOTAL_DISPLAY_GRAVITY_COUNT)
        mbVideoRemoteDisplayGravity = 0;
    
    NSString* aGravity[TOTAL_DISPLAY_GRAVITY_COUNT] = {kCAGravityResizeAspect, kCAGravityResize, kCAGravityResizeAspectFill, kCAGravityCenter};
    
    [mDisplayLayer setContentsGravity :aGravity[mbVideoRemoteDisplayGravity]];
    [mDisplayLayer setMasksToBounds: true];
}

- (void) AdjustContentGravity;
{
    mbVideoRemoteDisplayGravity++;
    [self ApplyContentGravity];
}

+ (void) PostAdjustContentGravity;
{
    if(mpVideoRemoteView == NULL)
        return;
    
    [mpVideoRemoteView AdjustContentGravity];
}

- (IBAction) OnDoubleTapView :(id) sender;
{
    [cVideoRemoteView PostAdjustContentGravity];
}



@end
