
//  Created by VaxSoft
//  Copyright © 2025 VaxSoft. All rights reserved.

#import "cVideoView.h"
#import "cVideoRemoteView.h"
#import "cVideoDeviceView.h"
#import "cDialPadView.h"

#import "cVaxPhoneSIP.h"
#import "cMainTabBarController.h"
#import "cCameraTorch.h"

@interface cVideoView ()
@end

static Boolean mbVideoShowDialpadWhenAppear = false;
static cVideoView* mpVideoView = NULL;
static NSMutableString* mpVideoViewStatusText = NULL;

@implementation cVideoView

- (void) viewDidLoad
{
    [super viewDidLoad];
    
    mpTorch = [cCameraTorch new];
    
    [mVideoCheckBox LoadCheckBox :nil :@"avVideoOFF" :@"avVideoON"];
    [mMuteMicCheckBox LoadCheckBox :nil :@"avMicOFF" :@"avMicON"];
    [mSpeakerPhoneCheckBox LoadCheckBox :nil :@"avPhoneSpkOFF" :@"avPhoneSpkON"];
    
    [mCheckBoxCamTorch LoadCheckBox :nil :@"aFlashCamOFF" :@"aFlashCamON"];
    [self HideBtnTorch :true];
}

- (void) didReceiveMemoryWarning
{
    [super didReceiveMemoryWarning];
}

/////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////

- (void) viewWillAppear:(BOOL)animated
{
    mpVideoView = self;
    
    [super viewWillAppear :animated];
    
    if(mbVideoShowDialpadWhenAppear)
    {
        return;
    }

    [self UpdateStates];
    [self HideBtnTorch :true];
    
    [self ShowStatusMsg :mpVideoViewStatusText];
    [[cVaxPhoneVideo GetSelf] OpenVideoDevice];
}

- (void) viewDidAppear:(BOOL)animated
{
    [super viewDidAppear :animated];
    
    if(mbVideoShowDialpadWhenAppear)
    {
        mbVideoShowDialpadWhenAppear = false;
        [self performSegueWithIdentifier: @"SegueDialPadView" sender: self];
        
        return;
    }
    
    [self ShowWithAutoHide];
}

- (void) viewWillDisappear:(BOOL)animated
{
    mpVideoView = NULL;
    
    if(![[cVaxPhoneGeneral GetSelf] IsLineConnected])
        [[cVaxPhoneVideo GetSelf] CloseVideoDevice];
    
    [super viewWillDisappear :animated];
}

/////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////

+ (void) OnStartedAPP;
{
    mpVideoViewStatusText = [NSMutableString new];
    [cDialPadView OnStartedAPP];
}

+ (void) ShowDialpadWhenAppear :(NSString*) sNoText;
{
    if(sNoText.length == 0)
    {
        mbVideoShowDialpadWhenAppear = false;
        return;
    }
    
    [cDialPadView PostPhoneNoText :sNoText];
    mbVideoShowDialpadWhenAppear = true;
}

/////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////

+ (void) PostInitialize;
{
    if(mpVideoView != NULL)
        [[cVaxPhoneVideo GetSelf] OpenVideoDevice];
}

+ (void) PostDialCallStarted;
{
    [cDialPadView PostDialCallStarted];
}

+ (void) PostDialCallFailed :(int) nStatusCode :(NSString*) sReasonPhrase;
{
    [cDialPadView PostDialCallFailed :nStatusCode :sReasonPhrase];
}

+ (void) ResetData;
{
    [mpVideoViewStatusText setString :@""];
}

+ (void) PostStatusText :(NSString*) sMsgText;
{
    [cDialPadView PostStatusText :sMsgText];
    
    [mpVideoViewStatusText setString :sMsgText];
    
    if(mpVideoView != NULL)
        [mpVideoView ShowStatusMsg :sMsgText];
}

+ (void) PostConnectedCall;
{
    [[cVaxPhoneVideo GetSelf] OpenVideoDevice];
    
    [cDialPadView PostConnectedCall];
    [cVideoRemoteView PostConnectedCall];
}

///////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////

+ (void) PostDisconnectedCall;
{
    cCameraTorch* pTorch = [cCameraTorch new];
    [pTorch SwitchTorchMode :false];
    
    [cDialPadView PostDisconnectedCall];
    [cVideoRemoteView PostDisconnectedCall];
}

///////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////

+ (void) PostRemoteFrameRGB :(char*) pFrameRGB :(int) nFrameSize :(int) nFrameWidth :(int) nFrameHeight;
{
    [cVideoRemoteView PostFrameRGB :pFrameRGB :nFrameSize :nFrameWidth :nFrameHeight];
}

+ (void) PostDeviceFrameRGB :(char*) pFrameRGB :(int) nFrameSize :(int) nFrameWidth :(int) nFrameHeight;
{
    [cVideoDeviceView PostFrameRGB :pFrameRGB :nFrameSize :nFrameWidth :nFrameHeight];
}

/////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////

- (void) ShowStatusMsg :(NSString*) sMsgText;
{
    [mStatusText setText :sMsgText];
}

- (void) UpdateStates;
{
    Boolean bMute = [[cVaxPhoneMicSpk GetSelf] IsMuteMic];
    [mMuteMicCheckBox SetState :bMute];
    
    Boolean bEnable = [[cVaxPhoneGeneral GetSelf] IsSpeakerPhone];
    [mSpeakerPhoneCheckBox SetState :bEnable];
    
    bEnable = [[cVaxPhoneVideo GetSelf] IsVideoEnabled];
    [mVideoCheckBox SetState :bEnable];
}

/////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////

- (void) touchesBegan :(NSSet *)touches withEvent:(UIEvent *)event
{
    [self ShowWithAutoHide];
    [cVideoDeviceView ShowView];
}

/////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////

- (IBAction) OnClickVideo :(id) sender;
{
    Boolean bEnable = [mVideoCheckBox GetState];
    [[cVaxPhoneVideo GetSelf] EnableVideo :bEnable];
    
    [self ShowWithAutoHide];
}

- (IBAction) OnClickVoiceChanger :(id) sender;
{
    Boolean bEnable = false;
    
    if([[cVaxPhoneVoiceChanger GetSelf] IsVoiceChangerEnabled])
        bEnable = false;
    else
        bEnable = true;
    
    [[cVaxPhoneVoiceChanger GetSelf] VoiceChanger: bEnable];
    [self ShowWithAutoHide];
}

- (IBAction) OnClickHangup :(id) sender;
{
    [[cVaxPhoneSIP GetSelf] DisconnectCall];
}

- (IBAction) OnClickMuteMic :(id) sender;
{
    Boolean bMute = [mMuteMicCheckBox GetState];
    [[cVaxPhoneMicSpk GetSelf] MuteMic :bMute];
    
    [self ShowWithAutoHide];
}

- (IBAction) OnClickSpeakerPhone :(id) sender;
{
    Boolean bEnable = [mSpeakerPhoneCheckBox GetState];
    [[cVaxPhoneGeneral GetSelf] SpeakerPhone :bEnable];
    
    [self ShowWithAutoHide];
}

/////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////

- (void) OnShowWithAutoHide;
{
    [mVideoCheckBox setHidden :true];
    [mMuteMicCheckBox setHidden :true];
    [mSpeakerPhoneCheckBox setHidden :true];
    
    [mBtnCamRotate setHidden: true];
    [self HideBtnTorch :true];
    
    [mBtnVoiceChanger setHidden: true];
    [mBtnHangup setHidden: true];
}

- (void) ShowWithAutoHide;
{
    [mVideoCheckBox setHidden :false];
    [mMuteMicCheckBox setHidden :false];
    [mSpeakerPhoneCheckBox setHidden :false];
    
    [mBtnCamRotate setHidden: false];
    [self HideBtnTorch :false];
    
    [mBtnVoiceChanger setHidden: false];
    [mBtnHangup setHidden: false];
    
    [self UpdateStates];
    
    [NSObject cancelPreviousPerformRequestsWithTarget:self selector:@selector(OnShowWithAutoHide) object:nil];
    [self performSelector :@selector(OnShowWithAutoHide) withObject:nil afterDelay:4.0];
}

/////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////

- (IBAction) OnClickLogout :(id)sender
{
    [[cVaxPhoneSIP GetSelf] DisconnectCall];
    [cMainTabBarController DismissMainTabBar];
}

- (IBAction) OnClickRotateCamera :(id)sender
{
    [[cVaxPhoneVideo GetSelf] RotateVideoDevice];
    [self ShowWithAutoHide];
}

/////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////

- (IBAction) OnClickCameraTorch :(id)sender
{
    [mpTorch SwitchTorchMode :[mCheckBoxCamTorch GetState]];
    [self ShowWithAutoHide];
}

- (void) HideBtnTorch :(Boolean) bHidden;
{
    if(![mpTorch IsTorchAvailable])
    {
        [mCheckBoxCamTorch setHidden: true];
        return;
    }

    [mCheckBoxCamTorch setHidden: bHidden];
    [mCheckBoxCamTorch SetState :[mpTorch IsTorchModeON]];
}


/////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////

- (IBAction) UnwindFromDialpadView :(UIStoryboardSegue *)segue;
{
}


@end
