
//  Created by VaxSoft
//  Copyright © 2025 VaxSoft. All rights reserved.

#import "cDataSQL.h"

#define DATABASE_NAME  "VaxPhone.db"

static NSMutableString* m_sPathDatabaseSQL = NULL;

@implementation cDataSQL

+ (void) CreateFileDB;
{
	// Get the path to the documents directory and append the databaseName
    
	NSArray* documentPaths = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES);
	NSString* documentsDir = [documentPaths objectAtIndex:0];
	
    NSString* sDatabasePath = [documentsDir stringByAppendingPathComponent: @DATABASE_NAME];
    
    if(m_sPathDatabaseSQL == NULL)
        m_sPathDatabaseSQL = [NSMutableString new];
    
    [m_sPathDatabaseSQL setString: sDatabasePath];
	
	// Create a FileManager object, we will use this to check the status
	// of the database and to copy it over if required
	NSFileManager* fileManager = [NSFileManager defaultManager];
	
	// Check if the database has already been created in the users filesystem
	Boolean success = [fileManager fileExistsAtPath: m_sPathDatabaseSQL];
	
    // Check if the SQL database has already been saved to the users phone, if not then copy it over
    // If the database already exists then return without doing anything
    
	if(success) return;
	
	// If not then proceed to copy the database from the application to the users filesystem
	
	// Get the path to the database in the application package
	
    NSString* databasePathFromApp = [[[NSBundle mainBundle] resourcePath] stringByAppendingPathComponent: @DATABASE_NAME];
	
	// Copy the database from the package to the users filesystem
	[fileManager copyItemAtPath:databasePathFromApp toPath: m_sPathDatabaseSQL error:nil];
}

+ (void) DeleteFileDB;
{
    NSArray* documentPaths = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES);
    NSString* documentDir = [documentPaths objectAtIndex:0];
    
    NSString* sDatabasePath = [documentDir stringByAppendingPathComponent: @DATABASE_NAME];
    
    NSFileManager* FileManager = [NSFileManager defaultManager];
    
    NSError *error;
    [FileManager removeItemAtPath:sDatabasePath error:&error];
}

- (id) init;
{	
    m_pCompiledStatement = NULL;
    m_pDatabase = NULL;
    
    self = [super init];
	return self;
}

- (void) dealloc;
{	
    [self CloseDatabase];
}

- (Boolean) OpenDatabase;
{
    return (sqlite3_open([m_sPathDatabaseSQL UTF8String], &m_pDatabase) == SQLITE_OK);
}

- (void) CloseDatabase;
{
    if(m_pDatabase == NULL)
        return;
    
    sqlite3_close(m_pDatabase);
    m_pDatabase = NULL;
}

- (id) GetColumnData :(sqlite3_stmt*) pCompiledStament :(int) nColNo;
{
    int nType = sqlite3_column_type(pCompiledStament, nColNo);
    
    if(nType == SQLITE_TEXT)
        return [NSString stringWithUTF8String:(char *)sqlite3_column_text(pCompiledStament, nColNo)];
    
    if(nType == SQLITE_INTEGER)
        return [NSNumber numberWithInt: sqlite3_column_int(pCompiledStament, nColNo)];
    
    if(nType == SQLITE_BLOB)
    {
        const void* pBlobData = sqlite3_column_blob(pCompiledStament, nColNo);
        int nBlobLength = sqlite3_column_bytes(pCompiledStament, nColNo);
        
        return [NSData dataWithBytes:pBlobData length:nBlobLength];
    }
    
    return NULL;
}

- (Boolean) StartQuery :(NSString*) pQuery :(NSArray*) pParam;
{
    if(sqlite3_prepare_v2(m_pDatabase, [pQuery UTF8String], -1, &m_pCompiledStatement, NULL) != SQLITE_OK)
        return false;
    
    for(int nParamCount = 0; nParamCount < [pParam count]; nParamCount++)
    {
        id iParam = [pParam objectAtIndex :nParamCount];
        
        if([iParam isKindOfClass:[NSString class]])
        {
            NSString* sParamText = (NSString*) iParam;
            sqlite3_bind_text(m_pCompiledStatement, nParamCount + 1, [sParamText UTF8String], -1, NULL);
        }
        
        else if([iParam isKindOfClass:[NSNumber class]])
        {
            NSNumber* pParamNumber = (NSNumber*) iParam;
            sqlite3_bind_int(m_pCompiledStatement, nParamCount + 1, (int) [pParamNumber integerValue]);
            
        }
        
        else if([iParam isKindOfClass :[NSData class]])
        {
            NSData* pParamData = (NSData*) iParam;
            sqlite3_bind_blob(m_pCompiledStatement, nParamCount + 1, [pParamData bytes], (int) [pParamData length], SQLITE_TRANSIENT);
        }
    }
    
    return TRUE;
}

- (Boolean) StartQuery :(NSString*) pQuery;
{
    if(sqlite3_prepare_v2(m_pDatabase, [pQuery UTF8String], -1, &m_pCompiledStatement, NULL) != SQLITE_OK)
        return false;
    
    return TRUE;
}

- (void) EndQuery;
{
    if(m_pCompiledStatement == NULL)
        return;
    
    sqlite3_finalize(m_pCompiledStatement);
    m_pCompiledStatement = NULL;
}


- (Boolean) GetRowData :(NSMutableArray*) pRowDB;
{
    if(m_pCompiledStatement == NULL)
        return false;
    
    if(sqlite3_step(m_pCompiledStatement) != SQLITE_ROW) 
        return false;
    
    for(int nColumnCount = 0 ;; nColumnCount++)
    {
        id ColumnData = [self GetColumnData: m_pCompiledStatement: nColumnCount];
        if(ColumnData == NULL) break;
        
        [pRowDB addObject: ColumnData];
    }
    
    return true;
}

- (Boolean) ShowRecord :(NSString*) pQuery;
{
    if(![self OpenDatabase])
    {
        return false;
    }
    
    [self StartQuery: pQuery];
    
    NSMutableArray* pRowArray = [NSMutableArray new];
    NSMutableString* pLog = [NSMutableString new];
    
    while(true)
    {
        [pRowArray removeAllObjects];
        
        if(![self GetRowData: pRowArray])
            break;
        
        for(int nCount = 0; nCount < [pRowArray count]; nCount++)
        {
            NSObject* pClass = [pRowArray objectAtIndex: nCount];
            
            if([pClass isKindOfClass: [NSString class]])
            {
                [pLog appendString :(NSString*) pClass];
            }
            
            if([pClass isKindOfClass: [NSNumber class]])
            {
                NSString* pString = [NSString stringWithFormat: @"%d", [(NSNumber*) pClass intValue]];
                [pLog appendString :(NSString*) pString];
            }
            
            [pLog appendString :(NSString*) @", "];
        }
        
        [pLog appendString :(NSString*) @"\n"];
        
    }
    
    NSLog(@"%@", pLog);
    
   // [pRowArray release];
   // [pLog release];
    
    [self EndQuery];
    [self CloseDatabase];
    
    return true;
}


/*
- (sqlite3_stmt*) ExecuteQuery :(NSString*) pQuery;
{
    //const char *sqlStatement = "SELECT * FROM Call_History_Contact";
    
    [self OpenDatabase];
    
    //const char *sqlStatement = "INSERT INTO Call_History_Contact (Contact_Name) VALUES ('Ahmad')";
    
    //sqlite3_stmt *compiledStatement;
    
    if(sqlite3_prepare_v2(m_pDatabase, [pQuery UTF8String], -1, &m_pCompiledStatement, NULL) != SQLITE_OK)
        return NULL;
    
    
    while(sqlite3_step(m_pCompiledStatement) == SQLITE_ROW) 
    {
        // Read the data from the result row
        NSString *aName = [NSString stringWithUTF8String:(char *)sqlite3_column_text(m_pCompiledStatement, 1)];
        
        //NSLog(aName);
    }
     
    
    [self CloseDatabase];
    
    
    return m_pCompiledStatement;
}
*/

/*
- (Boolean) ExecuteQuery :(NSString*) pQuery :(NSMutableArray*) pResult;
{
    //sqlite3_stmt *compiledStatement;
    
    if(sqlite3_prepare_v2(m_pDatabase, [pQuery UTF8String], -1, &m_pCompiledStatement, NULL) != SQLITE_OK)
        return FALSE;
    
    return TRUE;
}
 */



/*
-(void) readAnimalsFromDatabase 
{
	// Setup the database object
	sqlite3 *database;
	
	// Init the animals Array
	animals = [[NSMutableArray alloc] init];
	
	// Open the database from the users filessytem
	if(sqlite3_open([databasePath UTF8String], &database) == SQLITE_OK) 
    {
		// Setup the SQL Statement and compile it for faster access
		const char *sqlStatement = "select * from animals";
		sqlite3_stmt *compiledStatement;
		if(sqlite3_prepare_v2(database, sqlStatement, -1, &compiledStatement, NULL) == SQLITE_OK) 
 {
			// Loop through the results and add them to the feeds array
			while(sqlite3_step(compiledStatement) == SQLITE_ROW) {
				// Read the data from the result row
				NSString *aName = [NSString stringWithUTF8String:(char *)sqlite3_column_text(compiledStatement, 1)];
				NSString *aDescription = [NSString stringWithUTF8String:(char *)sqlite3_column_text(compiledStatement, 2)];
				NSString *aImageUrl = [NSString stringWithUTF8String:(char *)sqlite3_column_text(compiledStatement, 3)];
				
				// Create a new animal object with the data from the database
				Animal *animal = [[Animal alloc] initWithName:aName description:aDescription url:aImageUrl];
				
				// Add the animal object to the animals Array
				[animals addObject:animal];
				
				[animal release];
			}
		}
		// Release the compiled statement from memory
		sqlite3_finalize(compiledStatement);
		
	}
	sqlite3_close(database);
	
}
 */




@end
