
//  Created by VaxSoft
//  Copyright © 2025 VaxSoft. All rights reserved.

#import "cStoreRecents.h"
#import "cDataSQL.h"

#define TABLE_NAME                  @"TableRecents"

#define COLUMN_RECORD_ID            @"RecordId"
#define COLUMN_OUTBOUND_CALL_TYPE   @"OutboundCallType"
#define COLUMN_NAME                 @"Name"
#define COLUMN_PHONE_NO             @"PhoneNo"
#define COLUMN_START_TIME           @"StartTime"
#define COLUMN_END_TIME             @"EndTime"
#define COLUMN_DURATION             @"Duration"
#define COLUMN_DAY_NO               @"DayNo"
#define COLUMN_HISTORY_TYPE_ID      @"HistoryTypeId"

///////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////

@implementation cStoreDataRecents

- (id) init;
{
    self = [super init];
    return self;
}

- (void) dealloc;
{
    
}

@end

///////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////

@implementation cStoreRecents

- (id) init;
{
    mpDataSQL = [cDataSQL new];
    
    self = [super init];
    return self;
}

- (void) dealloc;
{
    mpDataSQL = nil;
}

///////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////

- (void) CreateTable;
{
    if(![mpDataSQL OpenDatabase])
        return;
    
    NSString* sQuery = [NSString stringWithFormat :@"CREATE TABLE IF NOT EXISTS '%@' ('%@' INTEGER PRIMARY KEY AUTOINCREMENT, '%@' INTEGER, '%@' TEXT, '%@' TEXT, '%@' INTEGER, '%@' INTEGER, '%@' INTEGER, '%@' INTEGER, '%@' INTEGER )", TABLE_NAME, COLUMN_RECORD_ID, COLUMN_OUTBOUND_CALL_TYPE,
                        COLUMN_NAME, COLUMN_PHONE_NO, COLUMN_START_TIME, COLUMN_END_TIME, COLUMN_DURATION, COLUMN_DAY_NO, COLUMN_HISTORY_TYPE_ID];
    
    if(![mpDataSQL StartQuery: sQuery])
    {
        [mpDataSQL CloseDatabase];
        return;
    }
    
    NSMutableArray* pRecordArray = [NSMutableArray new];
    [mpDataSQL GetRowData: pRecordArray];
    
    [mpDataSQL EndQuery];
    [mpDataSQL CloseDatabase];
}

- (long) AddCallHistory :(Boolean) bOutboundCallType :(NSString*) sName :(NSString*) sPhoneNo :(long) nStartTime :(long) nEndTime :(long) nDuration :(long) nDayNo :(long) nHistoryTypeId;
{
    if(![mpDataSQL OpenDatabase])
        return -1;
    
    NSString* sQuery = [NSString stringWithFormat :@"INSERT INTO %@ (%@, %@, %@, %@, %@, %@, %@, %@) VALUES (?, ?, ?, ?, ?, ?, ?, ?)",
                            TABLE_NAME, COLUMN_OUTBOUND_CALL_TYPE, COLUMN_NAME, COLUMN_PHONE_NO, COLUMN_START_TIME, COLUMN_END_TIME, COLUMN_DURATION, COLUMN_DAY_NO, COLUMN_HISTORY_TYPE_ID];
    
    NSArray* aParam = [NSArray arrayWithObjects: [NSNumber numberWithBool :bOutboundCallType], sName,
                       sPhoneNo, [NSNumber numberWithLong: nStartTime], [NSNumber numberWithLong: nEndTime],
                       [NSNumber numberWithLong: nDuration], [NSNumber numberWithLong: nDayNo],
                       [NSNumber numberWithLong: nHistoryTypeId], nil];
    
    if(![mpDataSQL StartQuery :sQuery :aParam])
    {
        [mpDataSQL CloseDatabase];
        return -1;
    }
    
    NSMutableArray* pRecordArray = [NSMutableArray new];
    [mpDataSQL GetRowData: pRecordArray];
    [mpDataSQL EndQuery];
    
    sQuery = [NSString stringWithFormat: @"SELECT MAX(%@) FROM %@", COLUMN_RECORD_ID, TABLE_NAME];
    
    if(![mpDataSQL StartQuery :sQuery])
    {
        [mpDataSQL CloseDatabase];
        return -1;
    }
    
    pRecordArray = [NSMutableArray new];
    [mpDataSQL GetRowData: pRecordArray];
    [mpDataSQL EndQuery];
    
    if([pRecordArray count] == 0)
    {
        [mpDataSQL CloseDatabase];
        return -1;
    }
    
    int nRecordId = [[pRecordArray objectAtIndex: 0] intValue];
    
    [mpDataSQL CloseDatabase];
    return nRecordId;
}

///////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////

- (NSArray*) GetCallHistoryAll;
{
    NSMutableArray* pRecordAll = [NSMutableArray new];
    
    if(![mpDataSQL OpenDatabase])
        return pRecordAll;
    
    NSString* sQuery = [NSString stringWithFormat :@"SELECT * FROM %@ ORDER BY %@ ASC", TABLE_NAME, COLUMN_DAY_NO];
    
    if(![mpDataSQL StartQuery: sQuery])
    {
        [mpDataSQL CloseDatabase];
        return pRecordAll;
    }
    
    while(true)
    {
        NSMutableArray* pRecord = [NSMutableArray new];
        
        if(![mpDataSQL GetRowData: pRecord])
            break;
        
        cStoreDataRecents* pData = [cStoreDataRecents new];
        
        pData->m_nRecordId = [pRecord[0] integerValue];
        pData->m_bOutboundCallType = [pRecord[1] boolValue];
        pData->m_sName = pRecord[2];
        pData->m_sPhoneNo = pRecord[3];
        pData->m_nStartTime = [pRecord[4] integerValue];
        pData->m_nEndTime = [pRecord[5] integerValue];
        pData->m_nDuration = [pRecord[6] integerValue];
        pData->m_nDayNo = [pRecord[7] integerValue];
        pData->m_nHistoryTypeId = [pRecord[8] integerValue];
        
        [pRecordAll addObject :pData];
    }
    
    [mpDataSQL EndQuery];
    [mpDataSQL CloseDatabase];
    
    return pRecordAll;
}

///////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////

- (void) RemoveCallHistoryAll;
{
    if(![mpDataSQL OpenDatabase])
        return;
    
    NSString* sQuery = [NSString stringWithFormat :@"DELETE FROM %@", TABLE_NAME];
    
    if (![mpDataSQL StartQuery: sQuery])
    {
        [mpDataSQL CloseDatabase];
        return;
    }
    
    NSMutableArray* pRecordArray = [NSMutableArray new];
    [mpDataSQL GetRowData: pRecordArray];
    
    [mpDataSQL EndQuery];
    [mpDataSQL CloseDatabase];
}

- (void) RemoveCallHistory :(long) nRecordId;
{
    if(![mpDataSQL OpenDatabase])
        return;
    
    NSString* sQuery = [NSString stringWithFormat :@"DELETE FROM %@ WHERE %@ = '%ld'", TABLE_NAME, COLUMN_RECORD_ID, nRecordId];
    
    if (![mpDataSQL StartQuery: sQuery])
    {
        [mpDataSQL CloseDatabase];
        return;
    }
    
    NSMutableArray* pRecordArray = [NSMutableArray new];
    [mpDataSQL GetRowData: pRecordArray];
    
    [mpDataSQL EndQuery];
    [mpDataSQL CloseDatabase];
}


///////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////

/*
- (long) GetCallCount;
{
    if(![mpDataSQL OpenDatabase])
        return 0;
    
    NSString* sQuery = [NSString stringWithFormat :@"SELECT COUNT(*) FROM %@", TABLE_NAME];
    
    if(![mpDataSQL StartQuery: sQuery])
    {
        [mpDataSQL CloseDatabase];
        return 0;
    }
    
    NSMutableArray* pRecordArray = [NSMutableArray new];
    [mpDataSQL GetRowData: pRecordArray];
    
    [mpDataSQL EndQuery];
    [mpDataSQL CloseDatabase];
    
    NSNumber* pNoCount = pRecordArray[0];
    return [pNoCount integerValue];
}
 */

@end


