
//  Created by VaxSoft
//  Copyright © 2025 VaxSoft. All rights reserved.

#import "cVaxPhoneSIP.h"
#import "cStoreVoiceCodecs.h"

#define VOICE_CODEC_NO_KEY          @"VaxVoiceCodecNo"
#define VOICE_CODEC_ENABLED_KEY     @"VaxVoiceCodecEnabled"

@implementation cStoreVoiceCodecs

- (id) init;
{
    m_pDataPRF = [cDataPRF new];
    
    maDefaultCodecNo = [NSArray arrayWithObjects:
                        [NSNumber numberWithLong :VAX_AUDIO_CODEC_G711U],
                        [NSNumber numberWithLong :VAX_AUDIO_CODEC_G711A],
                        [NSNumber numberWithLong :VAX_AUDIO_CODEC_G729],
                        [NSNumber numberWithLong :VAX_AUDIO_CODEC_GSM610],
                        [NSNumber numberWithLong :VAX_AUDIO_CODEC_ILBC], nil];
    
    maDefaultCodecEnabled[VAX_AUDIO_CODEC_G711U] = true;
    maDefaultCodecEnabled[VAX_AUDIO_CODEC_G711A] = true;
    maDefaultCodecEnabled[VAX_AUDIO_CODEC_G729] = true;
    maDefaultCodecEnabled[VAX_AUDIO_CODEC_GSM610] = true;
    maDefaultCodecEnabled[VAX_AUDIO_CODEC_ILBC] = true;
    
    self = [super init];
    return self;
}

- (void) dealloc;
{
    
}

//////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////

- (void) SetVoiceCodec :(NSArray*) aCodecNo;
{
    [m_pDataPRF WriteArray :VOICE_CODEC_NO_KEY :aCodecNo];
}

- (void) GetVoiceCodec :(NSMutableArray*) aCodecNo;
{
    NSArray* aReadCodecNo = [m_pDataPRF ReadArray :VOICE_CODEC_NO_KEY :maDefaultCodecNo];
    
    if(aReadCodecNo.count != VAX_AUDIO_CODEC_TOTAL)
    {
        aReadCodecNo = maDefaultCodecNo;
    }
    
    for(int nCount = 0; nCount < [aReadCodecNo count]; nCount++)
        [aCodecNo addObject :[aReadCodecNo objectAtIndex :nCount]];
}

//////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////

- (void) SetVoiceCodecEnabled :(long) nCodecNo :(Boolean) bEnable;
{
    NSString* sKey = [NSString stringWithFormat: @"%@:%ld", VOICE_CODEC_ENABLED_KEY, nCodecNo];
    [m_pDataPRF WriteBoolean :sKey :bEnable];
}

- (Boolean) GetVoiceCodecEnabled :(long) nCodecNo;
{
    NSString* sKey = [NSString stringWithFormat: @"%@:%ld", VOICE_CODEC_ENABLED_KEY, nCodecNo];
    return [m_pDataPRF ReadBoolean :sKey :maDefaultCodecEnabled[nCodecNo]];
}

/////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////

- (void) ResetData;
{
    [m_pDataPRF RemoveKey :VOICE_CODEC_NO_KEY];
    
    for(long nCodecNo = 0; nCodecNo < VAX_AUDIO_CODEC_TOTAL; nCodecNo++)
    {
        NSString* sKey = [NSString stringWithFormat: @"%@:%ld", VOICE_CODEC_ENABLED_KEY, nCodecNo];
        [m_pDataPRF RemoveKey :sKey];
    }
}

@end
