
//  Created by VaxSoft
//  Copyright © 2025 VaxSoft. All rights reserved.

#import <Foundation/Foundation.h>
#import "cVaxPushKit.h"

@implementation cVaxPushKit
@synthesize mPushKitDelegate;

- (id)initWith :(id) iDelegate;
{
    self = [super init];
    
    mPushKitDelegate = iDelegate;
    return self;
}

- (void) dealloc;
{
   
}

/////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////

- (void) Start;
{
    mPushRegistry = [[PKPushRegistry alloc] initWithQueue:dispatch_get_main_queue()];
    
    mPushRegistry.delegate = self;
    mPushRegistry.desiredPushTypes = [NSSet setWithObject:PKPushTypeVoIP];
    
    if (![mPushKitDelegate respondsToSelector:@selector(OnVaxTryingPushKit)])
        return;
    
    [self performSelector: @selector(OnVaxTryingPushKit:) withObject: self afterDelay: 0.0];
}

/////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////

- (void) OnVaxTryingPushKit :(cVaxPushKit*) pVaxPushKit;
{
    [mPushKitDelegate OnVaxTryingPushKit];
}

- (void) OnVaxSuccessPushKit :(NSString*) sToken;
{
    [mPushKitDelegate OnVaxSuccessPushKit :sToken];
}

/////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////

#pragma mark -pushkitDelegate

- (NSString *)ConstructDeviceToken:(NSData *) hDeviceToken;
{
    const char* pData = [hDeviceToken bytes];
    NSMutableString* pToken = [NSMutableString string];

    for (NSUInteger nCount = 0; nCount < [hDeviceToken length]; nCount++)
    {
        [pToken appendFormat:@"%02.2hhX", pData[nCount]];
    }

    return [pToken copy];
}

- (void)pushRegistry:(PKPushRegistry *)registry didUpdatePushCredentials:(PKPushCredentials *) hCredentials forType:(NSString *)type{
    
    if([hCredentials.token length] == 0)
    {
        NSLog(@"voip token NULL");
        
        if (![mPushKitDelegate respondsToSelector:@selector(OnVaxFailedPushKit)])
            return;
        
        [mPushKitDelegate OnVaxFailedPushKit];
        return;
    }
    
    NSString* sDeviceToken = [self ConstructDeviceToken: hCredentials.token];
            
    if (![mPushKitDelegate respondsToSelector:@selector(OnVaxSuccessPushKit:)])
        return;
    
    [self performSelector: @selector(OnVaxSuccessPushKit:) withObject: sDeviceToken afterDelay: 1.0];
}


- (void)pushRegistry:(PKPushRegistry *)registry didReceiveIncomingPushWithPayload:(PKPushPayload *)hPayload forType:(PKPushType)type withCompletionHandler:(void(^)(void))completion

{
    NSDictionary* pCallInfo = [hPayload.dictionaryPayload objectForKey:@"aps"];
    
    NSString* sCallerName = [pCallInfo objectForKey:@"CallerName"];
    NSString* sCallerId = [pCallInfo objectForKey:@"CallerId"];
    
    if(sCallerName == nil)
        sCallerName = @"";
    
    if(sCallerId == nil)
        sCallerId = @"";
    
    if (![mPushKitDelegate respondsToSelector:@selector(OnVaxNotificationPushKit::)])
        return;
    
    [mPushKitDelegate OnVaxNotificationPushKit :sCallerName :sCallerId];
}

@end
